using System;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Web.UI.WebControls;
using System.Web.SessionState;

namespace DatabaseProj
{
	/// <summary>
	/// A class containing static methods for tasks common to all pages.
	/// </summary>
	public class Common
	{
        /// <summary>
        /// Gets the connection string stored in the Web.Config file.
        /// </summary>
        internal static string ConnectionString
        {
            get { return ConfigurationSettings.AppSettings["connString"]; }
        }

		internal static Guid ourCompanyGuid()
		{
			return new Guid("114E036E-D1B1-4A6A-84BA-7E9B180AF027");
		}


        /// <summary>
        /// Gets the shopping cart table containing Books.
        /// </summary>
        /// <param name="session">The <see cref="HttpSession"/> of the user.</param>
        /// <returns>Returns a <see cref="DataTable"/> containing the book data.</returns>
        internal static DataTable GetBookShoppingCart(HttpSessionState session)
        {
            return (DataTable)session["BookPurchases"];
        }


        /// <summary>
        /// Gets the shopping cart table containing CDs.
        /// </summary>
        /// <param name="session">The <see cref="HttpSession"/> of the user.</param>
        /// <returns>Returns a <see cref="DataTable"/> containing the CD data.</returns>
        internal static DataTable GetCDShoppingCart(HttpSessionState session)
        {
            return (DataTable)session["CDPurchases"];
        }

        
        /// <summary>
        /// Runs a stored procedure on the SQL Server specified in the connection string.
        /// </summary>
        /// <param name="connectionString">The connection string to the SQL server.</param>
        /// <param name="storedProc">The name of the stored procedure on the SQL server.</param>
        /// <returns>Returns a <see cref="DataSet"/> containing possibly multiple tables.</returns>
        /// <exception cref="SqlException">Thrown by the SQL Server when an error occurs.</exception>
        internal static DataSet RunStoredProcedure(string connectionString, string storedProc, params SqlParameter[] args)
        {
            // initialize a bunch of SQL/ADO.NET objects
            SqlConnection cn  = new SqlConnection(connectionString);
            SqlCommand cmd    = new SqlCommand(storedProc, cn);
            cmd.CommandType   = CommandType.StoredProcedure;

            // add parameters
            foreach (SqlParameter arg in args)
                cmd.Parameters.Add(arg);

            SqlDataAdapter da = new SqlDataAdapter(cmd);
            DataSet data      = new DataSet();

            // Fill() could easily throw a SqlException
            da.Fill(data);
            return data;
        }

		/// <summary>
		/// Initializes the shopping cart and sets numerous session variables.
		/// </summary>
		public static void InitializeShoppingCart(HttpSessionState session)
		{
			// get an empty BookItem table
			string bookSql = "SELECT * FROM plincoln_billing.dbo.BookItem WHERE 1=2";
			SqlDataAdapter daBooks = new SqlDataAdapter(bookSql, Common.ConnectionString);
			SqlCommandBuilder cbBooks = new SqlCommandBuilder(daBooks);
			DataTable dtBooks = new DataTable("BooksItem");
			daBooks.Fill(dtBooks);
			session.Add("BookPurchases", dtBooks);
			session.Add("BookDataAdapter", daBooks);
            
			// get an empty CDItem table
			string cdSql = "SELECT * FROM plincoln_billing.dbo.CDItem WHERE 1=2";
			SqlDataAdapter daCDs = new SqlDataAdapter(cdSql, Common.ConnectionString);
			SqlCommandBuilder cbCDs = new SqlCommandBuilder(daCDs);
			DataTable dtCDs = new DataTable("CDItem");
			daCDs.Fill(dtCDs);
			session.Add("CDPurchases", dtCDs);
			session.Add("CDDataAdapter", daCDs);
		}
	}
}
