using System;
using System.Configuration;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

namespace DatabaseProj
{
	/// <summary>
	/// A simple web interface for the SQL database
	/// </summary>
	public class StartPage : System.Web.UI.Page
	{
        protected System.Web.UI.WebControls.Label Label1;
        protected System.Web.UI.WebControls.DataGrid dgBooks;
        protected System.Web.UI.WebControls.Label Label2;
        protected System.Web.UI.WebControls.Button btnOutOfStock;
        protected System.Web.UI.WebControls.Button btnProductsAfter;
        protected System.Web.UI.WebControls.TextBox txtMinAmount;
        protected System.Web.UI.WebControls.Button btnShowAll;
        protected System.Web.UI.WebControls.Button btnWarehouses;
        protected System.Web.UI.WebControls.TextBox txtWarehouses;
        protected System.Web.UI.WebControls.Button btnAuthorArtist;
        protected System.Web.UI.WebControls.DropDownList ddlProductTypes;
        protected System.Web.UI.WebControls.DropDownList ddlAuthorArtist;
        protected System.Web.UI.WebControls.Panel booksPanel;
        protected System.Web.UI.WebControls.Label lblMusicRowCount;
        protected System.Web.UI.WebControls.Panel musicPanel;
        protected System.Web.UI.WebControls.Label lblBookRowCount;
		protected System.Web.UI.WebControls.Button ButtonFindTitle;
		protected System.Web.UI.WebControls.TextBox TextBoxFindTitle;
		protected System.Web.UI.WebControls.Button ButtonFindAuthorMusician;
		protected System.Web.UI.WebControls.TextBox TextBoxAuthorMusician;
		protected System.Web.UI.WebControls.Button ButtonKeyword;
		protected System.Web.UI.WebControls.TextBox TextBoxKeyword;
        protected System.Web.UI.WebControls.DataGrid dgMusic;

        private void Page_Load(object sender, System.EventArgs e)
        {
            // Put user code to initialize the page here
            if (Page.IsPostBack) return;

            try
            {
                // show all products
                this.ShowProductData();

                // load the author names
                this.ddlAuthorArtist.Items.Clear();
                foreach (DataRow nameRow in this.GetAuthors().Rows)
                    this.ddlAuthorArtist.Items.Add(new ListItem(nameRow[0].ToString(), nameRow[0].ToString()));

            }
            catch (Exception any)
            {
                Response.Write("Error: " + any.Message);
            }
        }


        /// <summary>
        /// Displays all products in the inventory.
        /// </summary>
        private void ShowProductData()
        {
            try
            {
                // runs a stored procedure on the SQL server
                DataSet products = Common.RunStoredProcedure(Common.ConnectionString, "spGetAllProducts");

                // binds the returned data to the data grids
                this.BindData(products.Tables[0], products.Tables[1]);
            }
            catch (Exception any)
            {
                this.PrintText(Color.Red, "Error: {0}", any.Message);
            }
        }


        /// <summary>
        /// Displays products that are currently out of stock.
        /// </summary>
        private void ShowOutOfStockProducts()
        {
            try
            {
                // run a stored procedure that gets out of stock items
                DataSet products = Common.RunStoredProcedure(Common.ConnectionString, "spGetOutOfStockProducts");

                // bind the data to the grid
                this.BindData(products.Tables[0], products.Tables[1]);
            }
            catch (Exception any)
            {
                this.PrintText(Color.Red, "Error: {0}", any.Message);
            }
        }


        /// <summary>
        /// Given an integer in the textbox, finds all the products whose inventory is at least that number.
        /// </summary>
        private void ShowProductsAfterNumber()
        {
            try
            {
                // execute the SQL procedure
                int amount = int.Parse(this.txtMinAmount.Text);
                DataSet ds = Common.RunStoredProcedure(Common.ConnectionString, "spGetProductsAvailableAfter", new SqlParameter("@minCount", amount));

                // bind the data to the datagrid
                this.BindData(ds.Tables[0], ds.Tables[1]);
            }
            catch(FormatException)
            {
                this.PrintText(Color.Red, "Error: A numeric value is required in the textbox.");
            }
            catch(Exception any)
            {
                this.PrintText(Color.Red, "Error: {0}", any.Message);
            }
        }


        /// <summary>
        /// Finds all products that are stocked in more than N warehouses (specified in the textbox).
        /// </summary>
        private void ShowProductsInDistinctWarehouses()
        {
            try
            {
                // get the number of warehouses specified in the textbox
                int warehouses = int.Parse(this.txtWarehouses.Text);
                DataSet ds = Common.RunStoredProcedure(Common.ConnectionString, "spGetProductsInMoreWarehouses", new SqlParameter("@warehouses", warehouses));

                // bind the data to the datagrid
                this.BindData(ds.Tables[0], ds.Tables[1]);
            }
            catch(FormatException)
            {
                this.PrintText(Color.Red, "Error: A numeric value is required in the textbox.");
            }
            catch(Exception any)
            {
                this.PrintText(Color.Red, "Error: {0}", any.Message);
            }
        }


        private void ShowMusicByArtist(string artist)
        {
            try
            {
                // queries for only music data
                DataSet ds = Common.RunStoredProcedure(Common.ConnectionString, "spGetMusicByArtist", new SqlParameter("@artist", artist));
                this.BindData(null, ds.Tables[0]);
            }
            catch(Exception any)
            {
                this.PrintText(Color.Red, "Error: {0}", any.Message);
            }
        }

        private void ShowBooksByAuthor(string author)
        {
            try
            {
                // queries for only book data
                DataSet ds = Common.RunStoredProcedure(Common.ConnectionString, "spGetBooksByAuthor", new SqlParameter("@author", author));
                this.BindData(ds.Tables[0], null);
            }
            catch(Exception any)
            {
                this.PrintText(Color.Red, "Error: {0}", any.Message);
            }
        }

        private DataTable GetAuthors()
        {
            return Common.RunStoredProcedure(Common.ConnectionString, "spGetAuthorNames").Tables[0];
        }

        private DataTable GetArtists()
        {
            return Common.RunStoredProcedure(Common.ConnectionString, "spGetArtistNames").Tables[0];
        }

        private DataSet GetProductData()
        {
            // read in the stored connection string
            string connString = Common.ConnectionString;
            DataSet products = new DataSet();

            // try to run the SQL query (might throw an exception)
            products = Common.RunStoredProcedure(connString, "spGetAllProducts");
            return products;
        }

        private void BindData(DataTable books, DataTable music)
        {
            // don't show the books datagrid if no books data is passed
            if (books != null && books.Rows.Count > 0)
            {
                this.dgBooks.DataSource = books.DefaultView;
                this.dgBooks.DataBind();
                this.lblBookRowCount.Text = string.Format("{0} book(s) found", books.Rows.Count);
                this.booksPanel.Visible = true;
            }
            else
            {
                this.booksPanel.Visible = false;
            }

            // don't show the music datagrid if no music data is passed
            if (music != null && music.Rows.Count > 0)
            {
                this.dgMusic.DataSource = music.DefaultView;
                this.dgMusic.DataBind();
                this.lblMusicRowCount.Text = string.Format("{0} CD(s) found", music.Rows.Count);
                this.musicPanel.Visible = true;
            }
            else
            {
                this.musicPanel.Visible = false;
            }

            // store the tables in Session memory
            Session["Music"] = music;
            Session["Books"] = books;
        }

        private void PrintText(Color color, string formatText, params object[] args)
        {
            string messageText = string.Format(formatText, args);
            string outputText  = string.Format("<font color='{0}'>{1}</font>", color.Name, messageText);
            Response.Write(outputText);
        }

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
            this.btnShowAll.Click += new System.EventHandler(this.btnShowAll_Click);
            this.btnAuthorArtist.Click += new System.EventHandler(this.btnAuthorArtist_Click);
            this.ddlProductTypes.SelectedIndexChanged += new System.EventHandler(this.ddlProductTypes_SelectedIndexChanged);
            this.ButtonFindTitle.Click += new System.EventHandler(this.ButtonFindTitle_Click);
            this.ButtonFindAuthorMusician.Click += new System.EventHandler(this.ButtonFindAuthorMusician_Click);
            this.ButtonKeyword.Click += new System.EventHandler(this.ButtonKeyword_Click);
            this.dgMusic.SelectedIndexChanged += new System.EventHandler(this.dgMusic_SelectedIndexChanged);
            this.Load += new System.EventHandler(this.Page_Load);

        }
		#endregion

        private void btnOutOfStock_Click(object sender, System.EventArgs e)
        {
            this.ShowOutOfStockProducts();
        }

        private void btnProductsAfter_Click(object sender, System.EventArgs e)
        {
            this.ShowProductsAfterNumber();
        }

        private void btnShowAll_Click(object sender, System.EventArgs e)
        {
            this.ShowProductData();
        }

        private void btnWarehouses_Click(object sender, System.EventArgs e)
        {
            this.ShowProductsInDistinctWarehouses();
        }

        private void btnAuthorArtist_Click(object sender, System.EventArgs e)
        {
            string type = this.ddlProductTypes.SelectedValue;
            switch(type.ToUpper())
            {
                case "BOOKS": this.ShowBooksByAuthor(this.ddlAuthorArtist.SelectedValue);
                    break;
                case "MUSIC": this.ShowMusicByArtist(this.ddlAuthorArtist.SelectedValue);
                    break;
                default: this.PrintText(Color.Red, "BUG!! Unknown Product Type: {0}", type);
                    break;
            }
        }


        private void ddlProductTypes_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            DataTable namesTable;

            if (this.ddlProductTypes.SelectedValue.ToUpper() == "BOOKS")
            {
                // load the author names
                namesTable = this.GetAuthors();
            }
            else
            {
                // load the artist names
                namesTable = this.GetArtists();
            }

            // empty out the list
            this.ddlAuthorArtist.Items.Clear();

            // populate the drop-down list
            foreach (DataRow nameRow in namesTable.Rows)
                this.ddlAuthorArtist.Items.Add(new ListItem(nameRow[0].ToString(), nameRow[0].ToString()));
        }

        public void dgBooks_Command(Object sender, DataGridCommandEventArgs e)
        {
            DataTable shoppingCart = (DataTable) Session["shoppingCart"];
			
			int rowIndex = e.Item.ItemIndex;
            DataTable books = (DataTable)Session["books"];
			DataRow row = books.Rows[rowIndex];
			shoppingCart.ImportRow(row);
			Response.Redirect("ShoppingCart.aspx");
         
        }

        public void dgMusic_Command(Object sender, DataGridCommandEventArgs e)
        {
			DataTable shoppingCart = (DataTable) Session["shoppingCart"];

            int rowIndex = e.Item.ItemIndex;
            DataTable music = (DataTable)Session["music"];
			DataRow row = music.Rows[rowIndex];		
			shoppingCart.ImportRow(row);
			Response.Redirect("ShoppingCart.aspx");
                        
        }

		private void ButtonFindTitle_Click(object sender, System.EventArgs e)
		{
			DataSet ds = Common.RunStoredProcedure(Common.ConnectionString, "cgillum_inventory.dbo.spGetProductByTitle", 
																		new SqlParameter("@title", TextBoxFindTitle.Text));
			BindData(ds.Tables[0], ds.Tables[1]);
		}

		private void ButtonFindAuthorMusician_Click(object sender, System.EventArgs e)
		{
			DataSet books = Common.RunStoredProcedure(Common.ConnectionString, "cgillum_inventory.dbo.spGetBooksByAuthor", 
				new SqlParameter("@author", TextBoxAuthorMusician.Text));
			DataSet music = Common.RunStoredProcedure(Common.ConnectionString, "cgillum_inventory.dbo.spGetMusicByArtist", 
				new SqlParameter("@artist", TextBoxAuthorMusician.Text)); 
			BindData(books.Tables[0], music.Tables[0]);

		}

		private void ButtonKeyword_Click(object sender, System.EventArgs e)
		{
			DataSet ds = Common.RunStoredProcedure(Common.ConnectionString, "cgillum_inventory.dbo.spGetBooksByKeyword", 
				new SqlParameter("@keyword", TextBoxKeyword.Text));
			BindData(ds.Tables[0], null);
		}

        private void dgMusic_SelectedIndexChanged(object sender, System.EventArgs e)
        {
        
        }
	}
}
