using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Data.SqlClient;

namespace DatabaseProj
{
	/// <summary>
	/// Summary description for ChooseShipping.
	/// </summary>
	public class ChooseShipping : System.Web.UI.Page
	{
		protected System.Web.UI.WebControls.Button ButtonContinue;
		protected System.Web.UI.WebControls.Panel PanelContent;
		protected System.Web.UI.WebControls.DataGrid DataGridTest;
		protected System.Collections.Hashtable infoTable;
	
		private void Page_Load(object sender, System.EventArgs e)
		{
			DataTable dtCDs = (DataTable)Session["CDPurchases"];
			DataTable dtBooks = (DataTable)Session["BookPurchases"];
			infoTable = new Hashtable();
			ArrayList warehouses = getWarehouses(dtCDs, dtBooks);
			for(int i=0; i < warehouses.Count; i++)
			{
				generateShipTable((string)warehouses[i], dtBooks, dtCDs);
			}
			PanelContent.Controls.Add(new System.Web.UI.HtmlControls.HtmlGenericControl("hr"));
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.ButtonContinue.Click += new System.EventHandler(this.ButtonContinue_Click);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		//Add warehouses from books and CDs to warehouse list
		private ArrayList getWarehouses(DataTable books, DataTable CDs)
		{
			ArrayList warehouses = new ArrayList();
			getWarehouses(books, warehouses);
			getWarehouses(CDs, warehouses);
			return warehouses;
		}

		//Add warehouses from items which are new to a
		private void getWarehouses(DataTable items, ArrayList a)
		{
			for(int i=0; i < items.Rows.Count; i++)
			{
				string warehouse = items.Rows[i]["warehouse"].ToString();
				if(a.Contains(warehouse))
					continue;
				else
					a.Add(warehouse);
			}
			return;
		}

		private void generateShipTable(string warehouse, DataTable books, DataTable CDs)
		{	
			DataTable cd = pruneOnWarehouse(warehouse, CDs.Copy());
			DataTable book = pruneOnWarehouse(warehouse, books.Copy());
			
			PackageInfo pInfo = new PackageInfo();
			PanelContent.Controls.Add(new System.Web.UI.HtmlControls.HtmlGenericControl("hr"));
			PlaceHolder package = new PlaceHolder();
			
			//Add Label
			Label warehouseLabel = new Label();
			warehouseLabel.Text = "<u>Package Shipped from Warehouse " + warehouse + "<br><br>Contents:</u> ";
			package.Controls.Add(warehouseLabel);
			
			//Add Book Table
			if(book.Rows.Count > 0)
			{
				DataGrid dgBooks = new DataGrid();
				//Name
				BoundColumn bookName = new BoundColumn();
				bookName.DataField = "bookName";
				bookName.HeaderText = "Book Name";
				//Price
				BoundColumn price = new BoundColumn();
				price.DataField = "unitPrice";
				price.HeaderText = "Price";
				price.DataFormatString = "${0:f2}";
				//Quantity
				BoundColumn quantity = new BoundColumn();
				quantity.DataField = "quantity";
				quantity.HeaderText = "Quantity";

				dgBooks.Columns.Add(bookName);
				dgBooks.Columns.Add(price);
				dgBooks.Columns.Add(quantity);

				dgBooks.AutoGenerateColumns = false;
				dgBooks.DataSource = book.DefaultView;
				dgBooks.DataBind();
				package.Controls.Add(dgBooks);
			}			
			//Add CD Table
			if(cd.Rows.Count > 0)
			{
				//Name
				BoundColumn cdName = new BoundColumn();
				cdName.DataField = "CDName";
				cdName.HeaderText = "CD Name";
				//Price
				BoundColumn price = new BoundColumn();
				price.DataField = "unitPrice";
				price.HeaderText = "Price";
				price.DataFormatString = "${0:f2}";
				//Quantity
				BoundColumn quantity = new BoundColumn();
				quantity.DataField = "quantity";
				quantity.HeaderText = "Quantity";

				//package.Controls.Add(new System.Web.UI.HtmlControls.HtmlGenericControl("br"));
				DataGrid dgCDs = new DataGrid();
				dgCDs.Columns.Add(cdName);
				dgCDs.Columns.Add(price);
				dgCDs.Columns.Add(quantity);

				dgCDs.AutoGenerateColumns = false;
				dgCDs.DataSource = cd.DefaultView;
				dgCDs.DataBind();
				package.Controls.Add(dgCDs);
			}

			//Calculate weight and number of Items
			int weight = 0;
			int numItems = 0;
			for(int i=0; i<cd.Rows.Count; i++)
			{
				numItems += (int)cd.Rows[i]["quantity"];
				DataSet cdWeight = Common.RunStoredProcedure(Common.ConnectionString, "cgillum_inventory.dbo.spGetCDWeight",
								new SqlParameter("@CDID", cd.Rows[i]["CDID"].ToString()));
				weight += ((int)cdWeight.Tables[0].Rows[0]["weight"])*(int)cd.Rows[i]["quantity"];				
			}
			for(int i=0; i<book.Rows.Count;i++)
			{
				numItems += (int)book.Rows[i]["quantity"];
				DataSet bookWeight = Common.RunStoredProcedure(Common.ConnectionString, "cgillum_inventory.dbo.spGetBookWeight",
					new SqlParameter("@ISBN", book.Rows[i]["ISBN"].ToString()));
				weight += ((int)bookWeight.Tables[0].Rows[0]["weight"])*(int)book.Rows[i]["quantity"];
			}

			//Update pInfo
			pInfo.weight = weight;
			pInfo.numItems = numItems;

			//Special Instructions Box
			TextBox specialInstr = new TextBox();
			specialInstr.MaxLength = 100;
			pInfo.specialInstr = specialInstr;
			Label sLabel = new Label();
			sLabel.Text = "<br>Special Delivery Instructions:";
			package.Controls.Add(sLabel);
			package.Controls.Add(specialInstr);

			//Shipping Options//
			package.Controls.Add(new System.Web.UI.HtmlControls.HtmlGenericControl("br"));
			Label shippingOptions = new Label();
			shippingOptions.Text = "<u>Choose Shipping Option:</u>";
			package.Controls.Add(shippingOptions);

			//Get Cheapest Options Table
			DataSet cheapest = Common.RunStoredProcedure(Common.ConnectionString, "drashid_shipping.dbo.spCheapest", 
					new SqlParameter("@numItems", numItems), new SqlParameter("@weight", weight), 
					new SqlParameter("@destination", Session["destination"].ToString()), new SqlParameter("@warehouseID", warehouse));
			DataTable shipping = cheapest.Tables[0];
			shipping.Columns.Add("Arrival Date");

			//Update Drop Down Box and Date Column
			DropDownList shippingChoice = new DropDownList();			
			for(int i=0; i < shipping.Rows.Count;i++)
			{	
				shipping.Rows[i]["Arrival Date"] = (System.DateTime.Today.AddDays(double.Parse(shipping.Rows[i]["DeliveryTime"].ToString())).ToShortDateString()).ToString();
				string dropDownText = shipping.Rows[i]["ShippingMethod"].ToString() + ", " + shipping.Rows[i]["VendorName"].ToString();
				shippingChoice.Items.Add(new ListItem(dropDownText,i.ToString()));
			}

			pInfo.list = shippingChoice;
			pInfo.shipping = shipping;

			//Create Shipping Table
			DataGrid dgShipping = new DataGrid();			
			BoundColumn vendorName = new BoundColumn();  
			vendorName.HeaderText = "Vendor Name"; 
			vendorName.DataField = "vendorname";
			BoundColumn shippingMethod = new BoundColumn(); 
			shippingMethod.DataField = "shippingMethod"; 
			shippingMethod.HeaderText = "Shipping Method";
			BoundColumn pricingOption = new BoundColumn(); 
			pricingOption.HeaderText = "Pricing Option";
			pricingOption.DataField = "pricingoptiontype";
			BoundColumn priceCol = new BoundColumn();
			priceCol.DataField = "price";
			priceCol.HeaderText = "Price";
			priceCol.DataFormatString = "${0:f2}";
			BoundColumn arriveDate = new BoundColumn();
			arriveDate.HeaderText = "Arrival Date";
			arriveDate.DataField = "Arrival Date";

			dgShipping.Columns.Add(vendorName);
			dgShipping.Columns.Add(shippingMethod);
			dgShipping.Columns.Add(pricingOption);
			dgShipping.Columns.Add(priceCol);
			dgShipping.Columns.Add(arriveDate);
			dgShipping.AutoGenerateColumns = false;

			dgShipping.DataSource = shipping;
			dgShipping.DataBind();
			
			package.Controls.Add(dgShipping);
			package.Controls.Add(shippingChoice);

			infoTable.Add(warehouse, pInfo);

			//Add Placeholder to Control
			PanelContent.Controls.Add(package);	
			PanelContent.Controls.Add(new System.Web.UI.HtmlControls.HtmlGenericControl("br"));
		}

		private DataTable pruneOnWarehouse(string warehouse, DataTable item)
		{
			for(int i=0; i < item.Rows.Count;i++)
			{
				if(item.Rows[i]["warehouse"].ToString() != warehouse)
					item.Rows.RemoveAt(i);
			}
			return item;
		}

		private void ButtonContinue_Click(object sender, System.EventArgs e)
		{
			string packageSQL = "SELECT * FROM drashid_shipping.dbo.Packages WHERE 1=2";
			SqlDataAdapter daPackages = new SqlDataAdapter(packageSQL, Common.ConnectionString);
			SqlCommandBuilder cbPackages = new SqlCommandBuilder(daPackages);
			DataTable dtPackages = new DataTable("Packages");
			daPackages.Fill(dtPackages);

			double shippingCost = 0;
			foreach (object key in infoTable.Keys)
			{
				string warehouse = key.ToString();
				PackageInfo nextPackage = (PackageInfo)infoTable[key];	
				shippingCost += double.Parse(nextPackage.shipping.Rows[int.Parse(nextPackage.list.SelectedValue)]["Price"].ToString());
				DataRow package = dtPackages.NewRow();
				package["sourceWarehouseID"] = warehouse;
				package["weight"] = nextPackage.weight;
				package["NumberItems"] = nextPackage.numItems;
				package["shippingMethod"] = nextPackage.shipping.Rows[int.Parse(nextPackage.list.SelectedValue)]["shippingMethod"];
				package["shipVendorID"] = nextPackage.shipping.Rows[int.Parse(nextPackage.list.SelectedValue)]["vendorID"];
				package["DeliveryTime"] = nextPackage.shipping.Rows[int.Parse(nextPackage.list.SelectedValue)]["Arrival Date"];
				package["specialInstructions"] = nextPackage.specialInstr.Text;
				
				DataRow billTo = Common.RunStoredProcedure(Common.ConnectionString, "plincoln_billing.dbo.spGetCustomer",
					new SqlParameter("@custID", ((string) Session["customerID"]))).Tables[0].Rows[0];
				DataTable dtShipTo = (DataTable) Session["ShipTo"];
				DataRow shipTo = (dtShipTo == null) ? billTo : dtShipTo.Rows[0];

				package["DeliveryName"] = shipTo["firstName"].ToString() + " " + shipTo["lastName"].ToString();
				package["DeliveryContact"] = shipTo["phoneNumber"].GetType().ToString().Equals("System.DBNull") ? DBNull.Value : shipTo["phoneNumber"];
				package["DeliveryStreet"] = shipTo["streetAddress"].ToString();
				package["DeliveryCity"] = shipTo["city"].ToString();
				package["DeliveryState"] = shipTo["state"].ToString();
				package["DeliveryZip"] = shipTo["zipCode"].ToString();
				package["Status"] = "in progress";
				package["InvoiceID"] = ((DataTable)Session["Invoice"]).Rows[0]["invoiceID"];				
				package["TrackingNumber"] = Math.Abs((Guid.NewGuid().ToString()).GetHashCode());
				dtPackages.Rows.Add(package);
			}

			//showPackages(dtPackages);

			((DataTable)Session["Invoice"]).Rows[0]["shippingCost"] = shippingCost;
			((DataTable)Session["Invoice"]).Rows[0]["shippingMethod"] = "X";
			((DataTable)Session["Invoice"]).Rows[0]["expectedShipDate"] = System.DateTime.Today;

			Session.Add("PackageAdapter", daPackages);
			Session.Add("PackageTable", dtPackages);
			Response.Redirect("Payment.aspx");
		}

		private void showPackages(DataTable dtPackages)
		{
			DataGridTest.DataSource = dtPackages;
			DataGridTest.DataBind();

		}

		private class PackageInfo
		{
			public int weight;
			public int numItems;
			public DropDownList list;
			public DataTable shipping;
			public TextBox specialInstr;
		}
	}
}
