using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Data.SqlClient;

namespace DatabaseProj
{
	/// <summary>
	/// Summary description for invoice.
	/// </summary>
	public class Invoice : System.Web.UI.Page
	{
		protected System.Web.UI.WebControls.Label LabelTotal;
		protected System.Web.UI.WebControls.Label LabelShipping;
		protected System.Web.UI.WebControls.Label LabelTax;
		protected System.Web.UI.WebControls.Label LabelSubtotal;
		protected System.Web.UI.WebControls.DataGrid GridItemsPurchased;
		protected System.Web.UI.WebControls.Label LabelCCInfo;
		protected System.Web.UI.WebControls.Label LabelPaymentType;
		protected System.Web.UI.WebControls.Label LabelBillTo;
		protected System.Web.UI.WebControls.Label LabelShipTo;
		protected System.Web.UI.WebControls.Label LabelShippingType;
		protected System.Web.UI.WebControls.Label LabelShipDate;
		protected System.Web.UI.WebControls.Label LabelOrderDate;
		protected System.Web.UI.WebControls.DataGrid DataGridPackages;
		protected System.Web.UI.WebControls.Label LabelInvoiceName;
	
		private void Page_Load(object sender, System.EventArgs e)
		{
			string customerID = (string) Session["customerID"];
			if(customerID == null) 
			{
				Response.Clear();
				Response.Redirect("index.aspx");
			}

			string invoiceID = (string) Request["invoiceID"];
			if(invoiceID == null) 
			{
				Response.Redirect("CustomerInvoices.aspx");
			}

			DataSet invoice = Common.RunStoredProcedure(Common.ConnectionString, "plincoln_billing.dbo.spGetFullInvoicePackages",
				new SqlParameter("@customerID", customerID),
				new SqlParameter("@invoiceID", invoiceID));

			if(invoice.Tables[0].Rows.Count > 0) 
			{
				// access ok, invoice found
				
				// extract info
				DataRow mainInfo = invoice.Tables[1].Rows[0];
				DataTable purchases = invoice.Tables[2];
				DataRow billTo = invoice.Tables[3].Rows[0];
				DataRow shipTo = invoice.Tables[4].Rows[0];
				DataTable packages = invoice.Tables[5];

				this.LabelInvoiceName.Text	= invoiceID;

				// Display main info
				this.LabelOrderDate.Text	= (string) mainInfo["formOD"];
				//this.LabelShipDate.Text		= (string) mainInfo["formESD"];
				this.LabelSubtotal.Text		= (string) mainInfo["subTotal"];
				this.LabelTax.Text			= (string) mainInfo["tax"];
				this.LabelShipping.Text		= (string) mainInfo["shipCost"];
				//this.LabelShippingType.Text	= (string) mainInfo["shippingMethod"];
				this.LabelPaymentType.Text	= (string) mainInfo["paymentType"];
				this.LabelTotal.Text		= (string) mainInfo["totalCost"];

				if(! mainInfo["CCNumber"].GetType().ToString().Equals("System.DBNull"))
				{
					// A credit card was used, so display that information too

					string CCstr = (string) mainInfo["CCnumber"];

					// obscure the start of the CC
					if(CCstr.Length == 15)
						CCstr = "XXXXX XXX XXX " + CCstr.Substring(11, 4);
					else if(CCstr.Length == 16)
						CCstr = "XXXX XXXX XXXX " + CCstr.Substring(12, 4);

					LabelCCInfo.Visible = true;
					LabelCCInfo.Text = "<h4>Credit Card Information:</h4>" +
						"<TABLE id=\"Table1\" cellSpacing=\"3\" cellPadding=\"3\" border=\"0\">" +
						"<TR>" +
						"<TD><STRONG>Number:</STRONG></TD>" +
						"<TD>" + CCstr + "</TD>" +
						"</TR>" +
						"<TR>" +
						"<TD><STRONG>Exp. Date:</STRONG></TD>" +
						"<TD>" + ((string) mainInfo["CCeM"]) + "/" + ((string) mainInfo["CCeY"]) + "</TD>" +
						"</TR>" +
						"<TR>" +
						"<TD><STRONG>Type:</STRONG></TD>" +
						"<TD>" + ((string) mainInfo["CCType"]) + "</TD>" +
						"</TR>" +
						"</TABLE>";
				}

				// display BillTo Info
				string text = ((string) billTo["firstName"]) + " " + ((string) billTo["lastName"]) + "<br>" + // name
					((string) billTo["streetAddress"]) + "<br>"; // 1st address line
				if(! billTo["aptAddress"].GetType().ToString().Equals("System.DBNull"))
					text += ((string) billTo["aptAddress"]) + "<br>";  // 2nd address line
				text += ((string) billTo["city"]) + ", " + ((string) billTo["state"]) + " " + ((string) billTo["zipCode"]);
				if(! billTo["extZipCode"].GetType().ToString().Equals("System.DBNull"))
					text += "-" + ((string) billTo["extZipCode"]); // extended zip code
				text += "<br>";
				if(! billTo["phone"].GetType().ToString().Equals("System.DBNull"))
					text += "<br>Phone: " + ((string) billTo["phone"]); // formatted phone number
					
				LabelBillTo.Text = text;

				text = "";

				// display ShipTo Info
				text = ((string) shipTo["firstName"]) + " " + ((string) shipTo["lastName"]) + "<br>" + // name
					((string) shipTo["streetAddress"]) + "<br>"; // 1st address line
				if(! shipTo["aptAddress"].GetType().ToString().Equals("System.DBNull"))
					text += ((string) shipTo["aptAddress"]) + "<br>";  // 2nd address line
				text += ((string) shipTo["city"]) + ", " + ((string) shipTo["state"]) + " " + ((string) shipTo["zipCode"]);
				if(! shipTo["extZipCode"].GetType().ToString().Equals("System.DBNull"))
					text += "-" + ((string) shipTo["extZipCode"]); // extended zip code
				text += "<br>";
				if(! shipTo["phone"].GetType().ToString().Equals("System.DBNull"))
					text += "<br>Phone: " + ((string) shipTo["phone"]); // formatted phone number
					
				LabelShipTo.Text = text;

				// Display purchase info
				GridItemsPurchased.DataSource = purchases;
				GridItemsPurchased.DataBind();

				DataGridPackages.DataSource = packages;
				DataGridPackages.DataBind();
			}
			else
			{
				// invalid access permission or no such invoice
				Response.Redirect("CustomerInvoices.aspx");
			}

		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
