using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

namespace DatabaseProj
{
	/// <summary>
	/// Summary description for Products.
	/// </summary>
	public class Products : System.Web.UI.Page
	{
        protected System.Web.UI.WebControls.Label lblProductID;
        protected System.Web.UI.WebControls.Label lblProductNameType;
        protected System.Web.UI.WebControls.Label lblProductIDType;
        protected System.Web.UI.WebControls.Label lblCompanyType;
        protected System.Web.UI.WebControls.Label lblCompany;
        protected System.Web.UI.WebControls.Label lblCreatorType;
        protected System.Web.UI.WebControls.Label lblCategoryType;
        protected System.Web.UI.WebControls.Label lblCreators;
        protected System.Web.UI.WebControls.Label lblCategories;
        protected System.Web.UI.WebControls.Label lblPrice;
        protected System.Web.UI.WebControls.Label lblWeight;
        protected System.Web.UI.WebControls.Label lblDiscountAmount;
        protected System.Web.UI.WebControls.Label lblDiscountPercent;
		protected System.Web.UI.WebControls.Label LabelQuantity;
        protected System.Web.UI.WebControls.Label lblProductName;
    
        private void Page_Load(object sender, System.EventArgs e)
        {
            // ignore postback calls
            if (this.Page.IsPostBack) return;

            // read in the query string data
            if (this.Request["id"] == null || this.Request["table"] == null)
            {
                // if no id was given, indicate this to the user and show them their way home
                Response.Write("ID number and Table required.  Return to <a href=''>home</a>.");
            }
            else
            {
                int productID = int.Parse(this.Request["id"]);
                string productTable = this.Request["table"];

                try
                {
                    this.ShowProductData(productID, productTable);
                }
                catch( Exception any )
                {
                    Response.Write("Error: " + any.Message);
                }
            }
        }

        private void ShowProductData(int productID, string productTable)
        {
            DataSet data = Common.RunStoredProcedure(Common.ConnectionString, "spGetAllProductInformation", new SqlParameter("@id", productID), new SqlParameter("@table", productTable));
            DataRow item = data.Tables[0].Rows[0];
            string productIDType, productNameType, companyType, creatorType, categoryType;
			
			DataSet quantity = Common.RunStoredProcedure(Common.ConnectionString, "cgillum_inventory.dbo.spGetQuantityInStock", 
																							new SqlParameter("@productID", productID));
            switch(productTable.ToUpper())
            {
                case "BOOKS":
                    productIDType   = "ISBN";
                    productNameType = "Title";
                    companyType     = "Publisher";
                    creatorType     = "Author";
                    categoryType    = "Category";

                    break;
                case "MUSIC":
                    productIDType   = "ID";
                    productNameType = "Name";
                    companyType     = "RecordCompany";
                    creatorType     = "Artist";
                    categoryType    = "Genre";

//                    DataRow cd = data.Tables[0].Rows[0];
//
//                    // set all the labels accordingly
//                    this.lblProductIDType.Text   = "ID";
//                    this.lblProductID.Text       = cd["ID"].ToString();
//                    this.lblProductNameType.Text = "Name";
//                    this.lblProductName.Text     = cd["Name"].ToString();
//                    this.lblCompanyType.Text     = "Record Company";
//                    this.lblCompany.Text         = cd["Record Company"].ToString();
//                    this.lblCreatorType.Text     = "Artists";
//                    this.lblCategoryType.Text    = "Genres";
//
//                    // list stuff
//                    foreach (DataRow row in data.Tables[0].Rows)
//                    {
//                        // add all the distinct authors
//                        if (!this.lbCreators.Items.Contains(this.lbCreators.Items.FindByText(row["Artist"].ToString())))
//                            this.lbCreators.Items.Add(new ListItem(row["Artist"].ToString(), row["Artist"].ToString()));
//
//                        // add all the distinct categories
//                        if (!this.lbCategories.Items.Contains(this.lbCategories.Items.FindByText(row["Genre"].ToString())))
//                            this.lbCategories.Items.Add(new ListItem(row["Genre"].ToString(), row["Genre"].ToString()));
//                    }
                    break;
                default: PrintText(Color.Red, "Error: Unknown table '{0}'", productTable);
                    return;
            }

            // set all the labels accordingly
            this.lblProductIDType.Text   = productIDType;
            this.lblProductID.Text       = item[productIDType].ToString();
            this.lblProductNameType.Text = productNameType;
            this.lblProductName.Text     = item[productNameType].ToString();
            this.lblCompanyType.Text     = companyType;
            this.lblCompany.Text         = item[companyType].ToString();
            this.lblCreatorType.Text     = creatorType;
            this.lblCategoryType.Text    = categoryType;
			
			string quantityStr = quantity.Tables[0].Rows[0]["Quantity"].ToString();
			if(quantityStr != String.Empty)
				this.LabelQuantity.Text      = quantity.Tables[0].Rows[0]["Quantity"].ToString();
			else
				this.LabelQuantity.Text      = "0";

            ArrayList creators = new ArrayList();
            ArrayList categories = new ArrayList();

            // list stuff
            foreach (DataRow row in data.Tables[0].Rows)
            {
                // add all the distinct authors
                if (!creators.Contains(row[creatorType]))
                    creators.Add(row[creatorType]);
                
                // add all the distinct categories
                if (!categories.Contains(row[categoryType]))
                    categories.Add(row[categoryType]);
            }

            this.lblCreators.Text        = CreateStringList(creators);
            this.lblCategories.Text      = CreateStringList(categories);
            this.lblPrice.Text           = double.Parse(item["Price"].ToString()).ToString("F2");
            this.lblWeight.Text          = double.Parse(item["Weight"].ToString()).ToString("F2");;
            this.lblDiscountAmount.Text  = double.Parse(item["DiscountAmount"].ToString()).ToString("F2");;
            this.lblDiscountPercent.Text =(double.Parse(item["DiscountPercent"].ToString())*100.0).ToString("F2");
        }

        private void PrintText(Color color, string formatText, params object[] args)
        {
            string messageText = string.Format(formatText, args);
            string outputText  = string.Format("<font color='{0}'>{1}</font>", color.Name, messageText);
            Response.Write(outputText);
        }

        private static string CreateStringList(IList items)
        {
            string itemList = string.Empty;
            System.Text.StringBuilder sb = new System.Text.StringBuilder();
            for(int i=0;i<items.Count;i++)
            {
                if (i == 0) 
                    sb.Append(items[i]);
                else
                    sb.Append(", " + items[i]);
            }

            return sb.ToString();
        }


		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
