using System;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;

namespace phase2
{
	/// <summary>
	/// Useful methods for querying the database.
	/// </summary>
	public class dbquery
	{
		private string sqlConnectString;

		public dbquery(string connectionString)
		{
			sqlConnectString = connectionString;
		}

		public SqlConnection createConnection()
		{
			return new SqlConnection(sqlConnectString);
		}

		public bool executeQuery(string query)
		{
			return executeCommand(new SqlCommand(query));
		}

		public bool executeCommand(SqlCommand command)
		{
			SqlConnection connection = createConnection();
			connection.Open();
			SqlTransaction transaction = connection.BeginTransaction();
			command.Connection = connection;
			command.Transaction = transaction;

			int rowsAffected = 0;
			
			try 
			{
				rowsAffected = command.ExecuteNonQuery();
				transaction.Commit();
			}
			catch(Exception e)
			{
				error("SQL exception: " + e.ToString());
				try
				{
					transaction.Rollback();
				}
				catch (SqlException ex)
				{
					error("Rollback failed: " + ex.ToString());
				}
			}
			finally 
			{
				connection.Close();
			}

			return rowsAffected > 0;
		}

		public object scalarCommand(SqlCommand command) 
		{
			SqlConnection connection = createConnection();
			connection.Open();
			SqlTransaction transaction = connection.BeginTransaction();
			command.Connection = connection;
			command.Transaction = transaction;

			try 
			{
				object result = command.ExecuteScalar();
				transaction.Commit();
				return result;
			}
			catch(Exception e)
			{
				error("Exception: " + e.ToString());
				try
				{
					transaction.Rollback();
				}
				catch (SqlException ex)
				{
					error("Rollback failed: " + ex.ToString());
				}
			}
			finally 
			{
				connection.Close();
			}

			return null;
		}

		public void query2grid(string query, System.Web.UI.WebControls.BaseDataList grid)
		{
			command2grid(new SqlCommand(query), grid);
		}

		public void command2grid(SqlCommand command, System.Web.UI.WebControls.BaseDataList grid)
			{
			try 
			{
				// SQL connecty dealy
				SqlConnection connection = createConnection();
				connection.Open();
				command.Connection = connection;
				SqlDataReader reader = command.ExecuteReader();

				if (reader.HasRows)
				{
					grid.DataSource = reader;
					grid.DataBind();
					grid.Visible = true;
				} 
				else 
				{
					grid.Visible = false;
				}

				connection.Close();
			}
			catch (System.Data.SqlClient.SqlException) 
			{
				grid.Visible = false;
			}
		}

		public System.Web.UI.WebControls.Label errorLabel;
		protected void error(string text)
		{
			if (errorLabel != null)
				errorLabel.Text += "<p class='error'>" + text + "</p>\n";
		}
	}

	public class dbquery_billing : dbquery
	{
		public dbquery_billing(): base(ConfigurationSettings.AppSettings["BillingDatabase"]) {}
		public dbquery_billing(System.Web.UI.WebControls.Label errorLabel): this()
		{
			this.errorLabel = errorLabel;
		}

		public int getCustomerId(string login, string password)
		{
			SqlConnection connection = createConnection();

			string query = "SELECT customerid FROM login WHERE login = @login AND password = @password";

			SqlCommand command = new SqlCommand(query, connection);
			command.Parameters.Add("@login", SqlDbType.VarChar, 255).Value = login;
			command.Parameters.Add("@password", SqlDbType.VarChar, 255).Value = password;

			try 
			{
				connection.Open();
				object result = command.ExecuteScalar();
				if (result == null)
					return -1;
				else
					return (int) result;
			}
			catch (System.Data.SqlClient.SqlException) 
			{
				return -1;
			}
			finally 
			{
				connection.Close();
			}
		}

		public bool updateAddress(int id, string address1, string address2, string city, string state, string zip)
		{
			SqlCommand command = new SqlCommand(
				"UPDATE address\n" +
				"SET address1 = @address1, address2 = @address2, city = @city, state = @state, zip = @zip\n" +
				"FROM customer\n" +
				"WHERE customer.addressid = address.id AND customer.id = @id\n");

			command.Parameters.Add("@address1", SqlDbType.VarChar, 255).Value = address1;
			command.Parameters.Add("@address2", SqlDbType.VarChar, 255).Value = address2;
			command.Parameters.Add("@city", SqlDbType.VarChar, 255).Value = city;
			command.Parameters.Add("@state", SqlDbType.VarChar, 2).Value = state;
			command.Parameters.Add("@zip", SqlDbType.VarChar, 10).Value = zip;
			command.Parameters.Add("@id", SqlDbType.Int).Value = id;

			return executeCommand(command);
		}

		/// <summary>
		/// Add a new recipient to the database with the given name and address.
		/// </summary>
		/// <returns>customerid of the recipient</returns>
		public int addRecipient(string name, string address1, string address2, string city, string state, string zip) {
			string insertAddress =
				"INSERT INTO address (id, address1, address2, city, state, zip)\n" +
				"SELECT MAX(address.id) + 1, @address1, @address2, @city, @state, @zip\n" +
				"FROM   address\n";

			string insertCustomer =
				"INSERT INTO customer (id, name, addressid)\n" +
				"SELECT MAX(customer.id) + 1, @name, MAX(address.id)\n" +
				"FROM   address, customer\n";

			SqlCommand command = new SqlCommand(insertAddress + insertCustomer);

			command.Parameters.Add("@address1", SqlDbType.VarChar, 255).Value = address1;
			command.Parameters.Add("@address2", SqlDbType.VarChar, 255).Value = address2;
			command.Parameters.Add("@city", SqlDbType.VarChar, 255).Value = city;
			command.Parameters.Add("@state", SqlDbType.VarChar, 2).Value = state;
			command.Parameters.Add("@zip", SqlDbType.VarChar, 10).Value = zip;

			command.Parameters.Add("@name", SqlDbType.VarChar, 255).Value = name;

			SqlConnection connection = createConnection();
			connection.Open();
			SqlTransaction transaction = connection.BeginTransaction();
			command.Connection = connection;
			command.Transaction = transaction;

			try 
			{
				int rowsAffected = command.ExecuteNonQuery();

				if (rowsAffected > 0)
				{
					int id = (int) new SqlCommand("SELECT MAX(customer.id) FROM customer\n", connection, transaction).ExecuteScalar();

					transaction.Commit();
					return id;
				}
				else
					transaction.Rollback();
			}
			catch(Exception e)
			{
				error("Exception: " + e.ToString());
				try
				{
					transaction.Rollback();
				}
				catch (SqlException ex)
				{
					error("Rollback failed: " + ex.ToString());
				}
			}
			finally 
			{
				connection.Close();
			}

			return -1;
		}

		public bool updateCreditCard(int customerid, string cardnumber, string company, int expiremonth, int expireyear)
		{
			SqlCommand command = new SqlCommand(
				"UPDATE creditcard\n" +
				"SET cardnumber = @cardnumber, company = @company, expiremonth = @expiremonth, expireyear = @expireyear\n" +
				"WHERE customerid = @customerid\n");

			command.Parameters.Add("@cardnumber", SqlDbType.VarChar, 16).Value = cardnumber;
			command.Parameters.Add("@company", SqlDbType.VarChar, 20).Value = company;
			command.Parameters.Add("@expiremonth", SqlDbType.Int).Value = expiremonth;
			command.Parameters.Add("@expireyear", SqlDbType.Int).Value = expireyear;
			command.Parameters.Add("@customerid", SqlDbType.Int).Value = customerid;

			if (!executeCommand(command)) 
			{
				// Try creating a new credit card
				command.CommandText =
					"INSERT INTO creditcard (customerid, cardnumber, company, expiremonth, expireyear)\n" +
					"VALUES (@customerid, @cardnumber, @company, @expiremonth, @expireyear)\n";

				return executeCommand(command);
			} 
			else 
			{
				return true;
			}
		}


		/// <summary>
		/// Create a new invoice.
		/// </summary>
		/// <returns>invoiceid of new invoice</returns>
		public int addInvoice(int buyerid, int recipientid, string shippingmethod, string paymentmethod, string creditcardnumber, float paymentamount)
		{
			string query =
				"INSERT INTO invoice (id, buyerid, recipientid, orderdate, shippingmethod, paymentmethod, creditcardnumber, paymentamount)\n" +
				"SELECT MAX(id) + 1, @buyerid, @recipientid, GETDATE(), @shippingmethod, @paymentmethod, @creditcardnumber, @paymentamount\n" +
				"FROM   invoice\n" +
				"\n" +
				"SELECT MAX(id) FROM invoice\n";

			SqlCommand command = new SqlCommand(query);

			command.Parameters.Add("@buyerid", SqlDbType.Int).Value = buyerid;
			command.Parameters.Add("@recipientid", SqlDbType.Int).Value = recipientid;
			command.Parameters.Add("@shippingmethod", SqlDbType.VarChar, 20).Value = shippingmethod;
			command.Parameters.Add("@paymentmethod", SqlDbType.VarChar, 20).Value = paymentmethod;
			command.Parameters.Add("@creditcardnumber", SqlDbType.Char, 16).Value = (creditcardnumber == null || creditcardnumber == "") ? DBNull.Value : (object) creditcardnumber;
			command.Parameters.Add("@paymentamount", SqlDbType.Money).Value = paymentamount;

			SqlConnection connection = createConnection();
			connection.Open();
			SqlTransaction transaction = connection.BeginTransaction();
			command.Connection = connection;
			command.Transaction = transaction;

			try 
			{
				int invoicenumber = (int) command.ExecuteScalar();
				transaction.Commit();
				return invoicenumber;
			}
			catch(Exception e)
			{
				error("Exception: " + e.ToString());
				try
				{
					transaction.Rollback();
				}
				catch (SqlException ex)
				{
					error("Rollback failed: " + ex.ToString());
				}
			}
			finally 
			{
				connection.Close();
			}

			return -1;
		}

		public int addItem(int invoiceid, object productid, object producttype, object name, object quantity, object unitprice, object warehouse)
		{
			string query =
				"INSERT INTO item (id, invoiceid, productid, producttype, name, quantity, unitprice, deliverydate, warehouse)\n" +
				"SELECT MAX(id + 1), @invoiceid, @productid, @producttype, @name, @quantity, @unitprice, GETDATE() + 4, @warehouse\n" +
				"FROM   item\n" +
				"\n" +
				"SELECT MAX(id) FROM item\n";

			SqlCommand command = new SqlCommand(query);

			command.Parameters.Add("@invoiceid", SqlDbType.Int).Value = invoiceid;
			command.Parameters.Add("@productid", SqlDbType.VarChar, 13).Value = productid;
			command.Parameters.Add("@producttype", SqlDbType.VarChar, 4).Value = producttype;
			command.Parameters.Add("@name", SqlDbType.VarChar, 255).Value = name;
			command.Parameters.Add("@quantity", SqlDbType.Int).Value = quantity;
			command.Parameters.Add("@unitprice", SqlDbType.Money).Value = unitprice;
			command.Parameters.Add("@warehouse", SqlDbType.VarChar, 50).Value = warehouse;

			object result = this.scalarCommand(command);

			if (result != null)
				return (int) result;
			else
				return -1;
		}
		public bool addPriceAdjust(int invoiceid, int itemid, string type, string description, float adjustment)
		{
			string query =
				"INSERT INTO priceadjustment (invoiceid, itemid, type, description, adjustment)\n" +
				"VALUES (@invoiceid, @itemid, @type, @description, @adjustment)\n";

			SqlCommand command = new SqlCommand(query);

			command.Parameters.Add("@invoiceid", SqlDbType.Int).Value = invoiceid;
			command.Parameters.Add("@itemid", SqlDbType.Int).Value = (itemid < 0) ? System.DBNull.Value : (object) itemid;
			command.Parameters.Add("@type", SqlDbType.VarChar, 20).Value = type;
			command.Parameters.Add("@description", SqlDbType.VarChar, 255).Value = description;
			command.Parameters.Add("@adjustment", SqlDbType.Money).Value = adjustment;

			return this.executeCommand(command);
		}
	}

	public class dbquery_shipping : dbquery
	{
		public dbquery_shipping(): base(ConfigurationSettings.AppSettings["ShippingDatabase"]) {}
	}

	public class dbquery_inventory : dbquery
	{
		public dbquery_inventory(): base(ConfigurationSettings.AppSettings["InventoryDatabase"]) {}

		public DataTable getInventoryTable()
		{
			String query = "SELECT 'book' AS type, isbn AS id, title FROM Books UNION SELECT 'cd', cdid, albumName FROM CDs";

			DataSet ds = new DataSet("Inventory");

			SqlConnection connection = createConnection();

			connection.Open();
			new SqlDataAdapter(query, connection).Fill(ds, "inventory");
			connection.Close();

			return ds.Tables["inventory"];
		}

		public DataTable getShoppingCart(int customerid)
		{
			string query =
				"SELECT carts.quantity, products.type, id, title, price, discount, weight\n" +
				", carts.partnerCompany\n" +
				"FROM   ShoppingCarts AS carts,\n" +
				"       (SELECT 'book' AS type, isbn AS id, title\n" +
				"        FROM Books UNION SELECT 'cd', cdid, albumName FROM CDs) AS products,\n" +
				"       AllProducts\n" +
				"WHERE  carts.itemID = products.id AND carts.itemType = products.type\n" +
				"       AND carts.itemID = AllProducts.idNum AND carts.itemType = AllProducts.type\n" +
				"       AND carts.partnerCompany = AllProducts.partnerCompany\n" +
				"       AND custId = @customerid\n";

			DataSet ds = new DataSet();

			SqlConnection connection = createConnection();
			SqlCommand command = new SqlCommand(query, connection);
			command.Parameters.Add("@customerid", SqlDbType.Int).Value = customerid;
			
			connection.Open();
			new SqlDataAdapter(command).Fill(ds, "cart");
			connection.Close();

			return ds.Tables["cart"];
		}

		public int getShoppingCartSize(string custID)
		{
			SqlConnection connection = createConnection();

			string query = "select count(*) from shoppingCarts where custID='"+custID+"' group by custID";

			SqlCommand command = new SqlCommand(query, connection);

			try 
			{
				connection.Open();
				object result = command.ExecuteScalar();
				if (result == null)
					return 0;
				else
					return (int) result;
			}
			catch (System.Data.SqlClient.SqlException) 
			{
				return 0;
			}
			finally 
			{
				connection.Close();
			}

		}

		public bool emptyShoppingCart(int customerid)
		{
			string query = "UPDATE AllProducts\n" +
				"SET    quantity = AllProducts.quantity - ShoppingCarts.quantity\n" +
				"FROM   ShoppingCarts\n" +
				"WHERE  itemID = idNum AND type = itemType\n" +
				"       AND CustId = @customerid\n" +
				"\n" +
				/*
				"UPDATE ProductLocals\n" +
				"SET    quantity = ProductLocals.quantity - ShoppingCarts.quantity\n" +
				"FROM   ShoppingCarts\n" +
				"WHERE  itemID = idNum AND type = itemType\n" +
				"       AND CustId = @customerid\n" +
				"\n" +
				*/
				"DELETE FROM ShoppingCarts\n" +
				"WHERE CustId = @customerid\n";

			SqlCommand command = new SqlCommand(query);

			command.Parameters.Add("@customerid", SqlDbType.Int).Value = customerid;

			return this.executeCommand(command);
		}
	}
}
