using System; 
using System.Collections; 
using System.Web.UI; 
using System.Web.UI.WebControls; 
using System.Web.UI.HtmlControls; 
using System.Text; 

namespace prinsess_inventory.Classes 
{ 
	/// <summary> 
	/// Utility class for an ASP.NET page that preloads images that 
	/// are used for mouseovers. 
	/// 
	/// Be sure to notice that this code is provided as a technology sample, 
	/// 'as is' and no warranties are made by the authors. 
	/// 
	/// For questions and comments: i.j.m.c.schepers@wxs.nl, 
	///                             Fons.Sonnemans@reflectionit.nl 
	/// 
	/// </summary> 
	public class RollOverImageManager 
	{ 
		private ArrayList mImages; 

		public RollOverImageManager() 
		{ 
			mImages = new ArrayList(); 
		} 

		public void AddImage(string imageURL) 
		{ 
			mImages.Add(imageURL); 
		} 

		public void InitializeImage(Image control, string imageURLMouseOver) 
		{ 
			InitializeImage(control, imageURLMouseOver, control.ImageUrl); 
		} 

		public void InitializeImage(Image control, 
			string imageURLMouseOver, 
			string imageURLMouseOut) 
		{ 
			AddImage(imageURLMouseOver); 

			AddAttribute(control.Attributes, "onMouseOut", imageURLMouseOut); 
			AddAttribute(control.Attributes, "onMouseOver", imageURLMouseOver); 

		} 

		public void InitializeImage(HtmlImage control, string imageURLMouseOver) 
		{ 
			InitializeImage(control, imageURLMouseOver, control.Src); 
		} 

		public void InitializeImage(HtmlImage control, 
			string imageURLMouseOver, 
			string imageURLMouseOut) 
		{ 
			AddImage(imageURLMouseOver); 

			AddAttribute(control.Attributes, "onMouseOut", imageURLMouseOut); 
			AddAttribute(control.Attributes, "onMouseOver", imageURLMouseOver); 

		} 

		public void AddAttribute(AttributeCollection a, 
			string eventName, 
			string imageUrl) 
		{ 
			try 
			{ 
				a.Remove(eventName); 
			} 
			finally 
			{ 
				a.Add(eventName, "this.src='" + imageUrl + "'"); 
			} 
		} 

		/// <summary> 
		/// Preload the images 
		/// </summary> 
		public void LoadImages(Page webPage) 
		{ 
			// Create JavaScript 
			StringBuilder buildString = new StringBuilder(); 
			buildString.Append("\n<SCRIPT LANGUAGE='JavaScript'>\n"); 
			buildString.Append("<!--\n"); 
			buildString.Append("var aImages = new Array();\n"); 
			for(int i = 0; i < mImages.Count; i++) 
			{ 
				buildString.Append("aImages[" + 
					i + "] = new Image();\n"); 
				buildString.Append("aImages[" + 
					i + "].src = '" + 
					mImages[i].ToString() + "';\n"); 
			} 
			buildString.Append("//-->\n"); 
			buildString.Append("</script>\n"); 

			// Register Client Script 
			if (webPage == null) 
			{ 
				throw new ArgumentException( 
					"The Page object is invalid."); 
			} 
			if (webPage.Request.Browser.JavaScript == true) 
			{ 
				webPage.RegisterClientScriptBlock("ImageLoader", 
					buildString.ToString()); 
			} 
		} 

	} 
}