using System;
using System.Data;
using System.Collections;
using System.Data.SqlClient;

namespace Femron
{
	/// <summary>
	/// For use only in case of emergencies
	/// ShoppingCart represents the client-side version of the cart held in the DB
	/// </summary>
	public class ShoppingCart
	{
		private ArrayList itemIDs;
		private ArrayList itemTypes;
		private ArrayList itemNames;
		private ArrayList quantity;
		private ArrayList weight;
		private ArrayList CompanyName;
		private SqlConnection sqlConn;


		public ShoppingCart(string accountID, SqlConnection sqlConn)
		{
			this.itemIDs = new ArrayList();
			this.itemTypes = new ArrayList();
			this.itemNames = new ArrayList();
			this.quantity = new ArrayList();
			this.weight = new ArrayList();
			this.CompanyName = new ArrayList();
			this.sqlConn = sqlConn;

			// SQL Connection to get shopping cart information
			sqlConn.Open();
			SqlCommand sqlComCart = new SqlCommand("lidstrom_billing.dbo.sp_infoizeCart",sqlConn);
			sqlComCart.CommandType = CommandType.StoredProcedure;
			sqlComCart.Parameters.Add("@accountID",accountID);

			DataSet cartData = new DataSet(); //information about all of the items in the cart
			(new SqlDataAdapter(sqlComCart)).Fill(cartData); //fill the preceding data set
			
			DataSet weightData = new DataSet(); //return data set for the item weight query
			
			// SQL Connection to get item weights
			SqlCommand sqlComWeight = new SqlCommand("sultan_inventory.dbo.sp_getItemWeight",sqlConn);
			sqlComWeight.CommandType = CommandType.StoredProcedure;

			for(int i = 0; i < cartData.Tables[0].Rows.Count; ++i)
			{
				//for each type of item in the cart, add its quantity and identification information to cartData
				quantity.Add(cartData.Tables[0].Rows[i][0].ToString());
				itemNames.Add(cartData.Tables[0].Rows[i][1].ToString());
				itemTypes.Add(cartData.Tables[0].Rows[i][3].ToString().Substring(0,1));
				itemIDs.Add(cartData.Tables[0].Rows[i][4].ToString());
				CompanyName.Add(cartData.Tables[0].Rows[i][5].ToString()); 
				
				//prepare to query for the weight information of a given item
				sqlComWeight.Parameters.Clear();
				sqlComWeight.Parameters.Add("@itemID", this.itemIDs[i]);
				sqlComWeight.Parameters.Add("@itemType", this.itemTypes[i]);
				sqlComWeight.Parameters.Add("@CompanyName", this.CompanyName[i]);
				weightData.Clear();

				//retrieve the weight information for the item and compute the total weight of the total quantity
				(new SqlDataAdapter(sqlComWeight)).Fill(weightData);
				weightData = Login.runStoredProcedure(sqlConn.ToString(), sqlComWeight);
				double w = Convert.ToDouble(weightData.Tables[0].Rows[0].ItemArray[0].ToString());
				int q = Convert.ToInt32(quantity[i].ToString());

                //store the total weight of the quantity of the items
				weight.Add(w * q);
			}
			sqlConn.Close();
		}


		//returns an ArrayList of string triples
		// where each entry is:
		//{WAREHOUSEID, QUANTITYSHIPPED,ITEMID, TYPE, COMPANYNAME}
		public ArrayList getWarehouseList()
		{
			this.sqlConn.Open();
			int item = 0;
			ArrayList warehouses = new ArrayList();
			DataSet result = new DataSet();
			while(item < itemIDs.Count)
			{
				int q = Convert.ToInt32(quantity[item].ToString());

				// TRY TO FILL ITEM IN CART WITH SUPPLIES FROM ONE WAREHOUSE
				SqlCommand sqlComQ = new SqlCommand("sultan_inventory.dbo.sp_fillQuantity",sqlConn);
				sqlComQ.CommandType = CommandType.StoredProcedure;

				// Get the arguments for 
				sqlComQ.Parameters.Add("@itemID", itemIDs[item]);
				sqlComQ.Parameters.Add("@itemType", itemTypes[item]);
				sqlComQ.Parameters.Add("@quantity", quantity[item]);
				sqlComQ.Parameters.Add("@CompanyName", CompanyName[item]);
				
				result = new DataSet();
				(new SqlDataAdapter(sqlComQ)).Fill(result);

				// if warehouse with enough stock is found add that warehouse to list and move on to the next round!
				if(result.Tables[0].Rows.Count > 0)
				{
					string warehouseID = result.Tables[0].Rows[0][0].ToString();
					string quant = q.ToString();
					string itemID = (string)itemIDs[item];
					string itemName = (string)itemNames[item];
					string totalWeight = (Convert.ToDouble(weight[item].ToString()) * q).ToString();
					//								WAREHOUSE ID                   Quantity Shipped       Item ID             Item Name          total weight
					warehouses.Add(new string[] {warehouseID,quant,itemID,itemName, totalWeight});
				}
				else
				{
					//FIND AS FEW WAREHOUSES AS POSSIBLE TO FILL ITEM ORDER
					int split = q - 1;
					while(q > 0 && split > 0)
					{
						sqlComQ = new SqlCommand("sultan_inventory.dbo.sp_fillQuantityLimited",sqlConn);
						sqlComQ.CommandType = CommandType.StoredProcedure;

						sqlComQ.Parameters.Add("@itemID", itemIDs[item]);
						sqlComQ.Parameters.Add("@quantity", q);
						sqlComQ.Parameters.Add("@itemType", itemTypes[item]);
						sqlComQ.Parameters.Add("@quantityLimit", (split + 1));
						sqlComQ.Parameters.Add("@CompanyName", CompanyName[item]);

						result = new DataSet();
						(new SqlDataAdapter(sqlComQ)).Fill(result);

						if(result.Tables[0].Rows.Count > 0)
							for(int i = 0; i < result.Tables[0].Rows.Count && q != 0; ++i)
							{
								int temp = Convert.ToInt32(result.Tables[0].Rows[i][1].ToString());
								warehouses.Add(new string[] {result.Tables[0].Rows[i][0].ToString(),((q < temp) ? q.ToString() : temp.ToString()),itemIDs[item].ToString(),itemNames[item].ToString(),(Convert.ToDouble(weight[item].ToString()) * q).ToString()});
								q -= temp;
							}
						split = split - 1;
					}

					//if split is 0 and the desired amount has not been selected - then the order is impossible!!
					//throw out the precious, precious work done up to this point and give the customer the finger
					if(split == 0 && q > 0)
						return null;
				}
				item++;
			}
			this.sqlConn.Close();
			return warehouses;
		}

	}
}
