using System;
using System.Collections;
using System.Data;
using foofaraw.inventory;

namespace foofaraw.management
{
	public class ManagementQueryResult
	{
		public ArrayList books;
		public ArrayList music;
		public int increment_size;
		public int footer_display;
		public int num_books;
		public int num_music;

		public ManagementQueryResult(DataSet b, DataSet m, int increments)
		{
			increment_size = 10;
			footer_display = 10;
			ParseBooks(b,increments);
			ParseMusic(m,increments);
		}

		private void ParseBooks(DataSet book_data,int increments) 
		{
			books = new ArrayList();
			if(book_data == null)
				return;
			DataRowCollection results = book_data.Tables[0].Rows;				
			this.num_books = results.Count;
			if(this.increment_size*(increments-1)>=results.Count)
				return;
			int start = GetStart(results.Count,increments);
			for(int i=start; i< results.Count && i<(start+this.increment_size) &&
				this.increment_size*(increments-1)<results.Count; i++) 
			{
				DataRow row = results[i];
				this.books.Add(new ManagementBook(Convert.ToInt64(row["isbn"]),
					(string)row["title"],
					GetAuthors(Convert.ToInt64(row["isbn"])),
					(Decimal)row["price"],
					(int)row["availible"],
					GetBooksSold(Convert.ToInt64(row["isbn"]))));    
			}
		}

		private void ParseMusic(DataSet music_data,int increments)
		{
			music = new ArrayList();
			if(music_data == null)
				return;
			DataRowCollection results = music_data.Tables[0].Rows;	
			this.num_music = results.Count;
			if(this.increment_size*(increments-1)>=results.Count)
				return;
			int start = GetStart(results.Count,increments);
			for(int i=start; i<results.Count && i<(start+this.increment_size) &&
				this.increment_size*(increments-1)<results.Count; i++) 
			{
				DataRow row = results[i];
				this.music.Add(new ManagementMusic(Convert.ToInt64(row["cdid"]),
					(string)row["title"],
					GetArtists(Convert.ToInt64(row["cdid"])),
					(Decimal)row["price"],
					(int)row["availible"],
					GetMusicSold(Convert.ToInt64(row["cdid"]))));     
			}
		}

		private ArrayList GetAuthors(Int64 isbn) 
		{
			ManagementSearch search = new ManagementSearch();
			DataTable results = search.SearchForBookAuthors(isbn).Tables[0];
			ArrayList authors = new ArrayList();
			foreach(DataRow row in results.Rows)
				authors.Add((string)row["name"]);
			return authors;
		}

		private ArrayList GetArtists(Int64 cdid) 
		{
			ManagementSearch search = new ManagementSearch();
			DataTable results = search.SearchForMusicArtists(cdid).Tables[0];
			ArrayList artists = new ArrayList();
			foreach(DataRow row in results.Rows)
				artists.Add((string)row["name"]);
			return artists;
		}

		private int GetBooksSold(Int64 isbn) 
		{
			ManagementSearch search = new ManagementSearch();
			DataTable results = search.GetNumberOfBooksSold(isbn).Tables[0];
			if(results.Rows[0]["sold"] is DBNull)
				return 0;
			else
				return Convert.ToInt32(results.Rows[0]["sold"]);
		}

		private int GetMusicSold(Int64 cdid) 
		{
			ManagementSearch search = new ManagementSearch();
			DataTable results = search.GetNumberOfMusicSold(cdid).Tables[0];
			if(results.Rows[0]["sold"] is DBNull)
				return 0;
			else
				return Convert.ToInt32(results.Rows[0]["sold"]);
		}


		public string CreateBookResults(int increments, string type, string search) 
		{			
			string html = "<div class=\"searchheader\">Books</div>";
			if(this.books.Count == 0)
				return html + NoResults();
			foreach(ManagementBook b in this.books) 
			{
				ManagementSearchResultTemplate template = new ManagementSearchResultTemplate();	
				template.InsertItemLeft("ISBN",b.isbn.ToString());
				template.InsertItemLeft("Title",b.title);
				template.InsertItemLeft("Author(s)",b.authors);
				template.InsertItemRight("Unit Price","$"+Decimal.Round(b.price,2).ToString());
				template.InsertItemRight("Availible",b.availible.ToString());
				template.InsertItemRight("Total Sold",b.sold.ToString());
				template.InsertItemRight("Revenue","$"+(Decimal.Round(b.price,2)*b.sold));
				html += template.create() + "\n";
			}      		
			return html + GetFooter(increments,type,search,this.num_books);
		}

		public string CreateMusicResults(int increments, string type, string search) 
		{
			string html = "<div class=\"searchheader\">Music</div>";
			if(this.music.Count == 0)
				return html + NoResults();
			foreach(ManagementMusic m in this.music) 
			{
				ManagementSearchResultTemplate template = new ManagementSearchResultTemplate();	
				template.InsertItemLeft("CDID",m.cdid.ToString());
				template.InsertItemLeft("Title",m.title);
				template.InsertItemLeft("Artist(s)",m.artists);
				template.InsertItemRight("Unit Price","$"+Decimal.Round(m.price,2).ToString());
				template.InsertItemRight("Availible",m.availible.ToString());
				template.InsertItemRight("Total Sold",m.sold.ToString());
				template.InsertItemRight("Revenue","$"+(Decimal.Round(m.price,2)*m.sold));
				html += template.create() + "\n";
			}      		
			return html + GetFooter(increments,type,search,this.num_music);
		}

		private string NoResults()
		{
			return "Search returned no results";
		}


		private int GetStart(int num_items, int increments) 
		{
			if(increments <= 0)
				return 0;
			else if(num_items < (increments-1) * this.increment_size)
				return 0;
			else
				return (increments-1) * this.increment_size;
		}

		public string GetFooter(int increments, string type, string search,int total_size) 
		{
			int upper;
			if(increments*this.increment_size<total_size)
				upper = increments*this.increment_size;
			else
				upper = total_size;
			string html = "<div class=\"searchfooter\"> ";
			html += "Viewing " + GetStart(total_size,increments) + " to " +
				upper + " of " + total_size + " results.<br />";
			if(increments > 1 && GetStart(total_size,increments)!=0)
				html += CreateLink(increments-1,type,search," &lt; Previous ");
			if(increments * this.increment_size < total_size)
				html += CreateLink(increments+1,type,search," Next &gt; ");
			return html + " </div>";
		}

		private string CreateLink(int increment, string type, string search, string text) 
		{
			return "<a href=\"manage_inventory.aspx?" + type + "=" + search + 
				"&count=" + increment +"\">" + text + "</a>";
		}
	}
}
