using System;
using SqlRequest;
using System.Data;

namespace foofaraw.management
{
	/// <summary>
	/// Summary description for ShippingManager.
	/// </summary>
	public class ShippingManager
	{
		public string GenerateResults(string city) 
		{
			ShippingSearch search = new ShippingSearch();
			DataTable shipping_methods = search.SearchForShippingMethodsByCity(city).Tables[0];
			string html = GetHeader(city);
			html += "<p>The following shipping methods are availible for this city.</p>";
			foreach(DataRow row in shipping_methods.Rows) 
			{
				string method = row["method_name"].ToString();
				html += "<div class=\"methodresult\">";
				html += "<div class=\"methodname\">" + method + "</div>";
				html += GetMethodData(method,city);
                html += "</div>\n";
			}
			return html;
		}

		private string GetMethodData(string method, string city) 
		{
			string html = string.Empty;
			ShippingSearch search = new ShippingSearch();
			DataTable vendors = search.SearchForVendordsByCityAndMethod(city,method).Tables[0];			
			DataTable number_shipped = search.SearchForPackagesByMethodName(city,method).Tables[0];
			html += "<div class=\"prices\">Method used " + number_shipped.Rows[0]["total"] + " times.</div>" +
				"<p\nAvailible Vendors and Pricing: <br/>\n";
			foreach(DataRow row in vendors.Rows) 
			{
				string vendor = row["vendor_name"].ToString();
				int vendor_id = Convert.ToInt32(row["vendor_id"]);
				html += "<h4>" + vendor + "</h4>";
				html += "<div class=\"prices\">";
				html += GetFixedPriceOptions(vendor_id,city) + "<br/><br/>";
				html += GetWeightPricing(vendor_id,city) + "<br/>";
				html += GetNumberPricing(vendor_id,city) + "<br/>";
				html += "</div>";
			}
			return html + "</p>";
		}

		private string GetFixedPriceOptions(int vendor, string city) 
		{
			ShippingSearch search = new ShippingSearch();
			DataTable price = search.SearchForFixedPriceOptions(vendor,city).Tables[0];
			string html = "<b>Fixed Price Options: </b>";
			foreach(DataRow row in price.Rows)  
			{
				html += Decimal.Round((Decimal)row["price"],2) + ", ";
			}
			html = html.Remove(html.LastIndexOf(","),2);
			return html;
		}

		private string GetWeightPricing(int vendor, string city) 
		{
			ShippingSearch search = new ShippingSearch();
			DataTable price = search.SearchForWeightPriceOptions(vendor,city).Tables[0];
			string html = "<b>Weight Price Options: </b><br/>";
			html += "<table width=\"350\">\n";
			foreach(DataRow row in price.Rows)  
			{
				html += "<tr><td width=\"250\">";
				html += row["weight_bracket_min"] + "lbs. to " + row["weight_bracket_max"] + "lbs.</td>" +
					"<td> $" + Decimal.Round((Decimal)row["price"],2);
				html +="</td></tr>\n";
			}
			return html + "</table>";
		}

		private string GetNumberPricing(int vendor, string city) 
		{
			ShippingSearch search = new ShippingSearch();
			DataTable price = search.SearchForNumPriceOptions(vendor,city).Tables[0];
			string html = "<b>Quantity Price Options: </b><br/>";
			html += "<table width=\"350\">\n";
			foreach(DataRow row in price.Rows)  
			{
				html += "<tr><td width=\"250\">";
				html += row["num_items_min"] + " items to " + row["num_items_max"] + " items</td>" +
					"<td>$" + Decimal.Round((Decimal)row["price"],2);
				html +="</td></tr>\n";
			}
			return html + "</table>";
		}

		private string GetHeader(string city) 
		{
			return "<div class=\"itembox\"><div class=\"titleitem\">Shipping for the city of " + city + "</div></div>\n";
		}

		private string GetError(string error) 
		{
			return "<div class=\"error\">There was an error implementing the request: " + error + "</div>\n";
		}

	}

	public class ShippingSearch
	{
		private SQLServer server = StaticServer.TimP;

		public DataSet SearchForShippingMethodsByCity(string city) 
		{			
			string sqlStr = "SELECT DISTINCT method_name " +
				"FROM Shipping_Methods sm " +
				"WHERE sm.method_available = 1 AND method_id NOT IN (SELECT method_id  FROM Cities c, Restricted_Cities rc " +
				"WHERE c.city_id = rc.city_id AND c.city_name = '" + city + "')";
			return ProcessQuery(sqlStr);
		}

		public DataSet SearchForPackagesByMethodName(string city, string method) 
		{			
			string sqlStr = "SELECT count(p.pkg_tracking_id) as total " +
				"FROM Packages p, Shipping_Methods sm " +
				"WHERE sm.method_available = 1 AND p.shipping_method_id = sm.method_id " +
				"AND sm.method_name = '" + method + "' and p.shipping_city = '" + city + "'";
			return ProcessQuery(sqlStr);
		}

		public DataSet SearchForVendordsByCityAndMethod(string city, string method) 
		{			
			string sqlStr = "SELECT sv.vendor_id, vendor_name " +
				"FROM Shipping_Vendors sv, Shipping_Methods sm " +
				"WHERE sm.method_available = 1 AND sm.vendor_id = sv.vendor_id AND sm.method_name = '" + method + "' " +
				"AND sm.method_id NOT IN (SELECT method_id  FROM Cities c, Restricted_Cities rc  " +
				"WHERE c.city_id = rc.city_id AND c.city_name = '" + city + "')";
			return ProcessQuery(sqlStr);
		}	

		public DataSet SearchForFixedPriceOptions(int vendor, string city) 
		{			
			string sqlStr = "SELECT fpo.price " +
				"FROM Fixed_Pricing_Options fpo, Shipping_Methods sm " +
				"WHERE sm.method_available = 1 AND sm.method_id = fpo.method_id AND sm.vendor_id = " + vendor + " " +
				"AND sm.method_id NOT IN (SELECT method_id  FROM Cities c, Restricted_Cities rc " +
				"WHERE c.city_id = rc.city_id AND c.city_name = '" + city + "')";
			return ProcessQuery(sqlStr);
		}

		public DataSet SearchForWeightPriceOptions(int vendor, string city) 
		{			
			string sqlStr = "SELECT weight_bracket_min, weight_bracket_max, price, sm.method_id " +
				"FROM Weight_Pricing_Options wpo, Shipping_Methods sm " +
				"WHERE sm.method_available = 1 AND sm.method_id = wpo.method_id AND sm.vendor_id = " + vendor + " " +
				"AND sm.method_id NOT IN (SELECT method_id  FROM Cities c, Restricted_Cities rc " +
				"WHERE c.city_id = rc.city_id AND c.city_name = '" + city + "')";
			return ProcessQuery(sqlStr);
		}

		public DataSet SearchForNumPriceOptions(int vendor, string city) 
		{			
			string sqlStr = "SELECT num_items_min, num_items_max, price, sm.method_id " +
				"FROM Num_Items_Pricing_Options npo, Shipping_Methods sm " +
				"WHERE sm.method_available = 1 AND sm.method_id = npo.method_id AND sm.vendor_id = " + vendor + " " +
				"AND sm.method_id NOT IN (SELECT method_id  FROM Cities c, Restricted_Cities rc " +
				"WHERE c.city_id = rc.city_id AND c.city_name = '" + city + "')";
			return ProcessQuery(sqlStr);
		}


		private DataSet ProcessQuery(string query) 
		{
			SQLRequest request = new SQLRequest(this.server, query);          
			return request.Invoke();
		}
	}
}
