using System;
using System.Data;
using SqlRequest;
using System.Text.RegularExpressions;

namespace foofaraw.inventory
{
	/// <summary>
	/// Summary description for Search.
	/// </summary>
	public class InventorySearch
	{
		private SQLServer server = StaticServer.CaseyH;
		private string standard_book_select;
		private string standard_book_group;
		private string standard_music_select;
		private string standard_music_group;
			
		public InventorySearch()
		{
			standard_book_select = "select distinct isbn,title,publishers.name AS publisher,date,price,weight,book_cat.name AS category,sum(availible) AS availible, discounts.name as disc_type, expire as disc_expire, percent_off as disc_percent, company ";
            standard_book_group = "group by isbn,title,publishers.name,date,price,weight,book_cat.name,discounts.name,expire,percent_off, company ";
			standard_music_select = "select distinct cdid,title,record_companies.name AS recordco,date,price,weight,music_genre.name AS genre,sum(availible) as availible, discounts.name as disc_type, expire as disc_expire, percent_off as disc_percent, company ";
			standard_music_group = "group by cdid,title,record_companies.name,date,price,weight,music_genre.name,discounts.name,expire,percent_off, company ";
		}

		public DataSet SearchForBookByTitle(string title) 
		{			
			string sqlStr = standard_book_select + 
				"from books,publishers,book_cat,book_inv,discounts " +
				"where publisher=publishers.id " +
				"and discounts.id=discount " +
				"and book_cat.id=category " +
				"and book_inv.bookid=isbn " +
				"and title like '" + FormatInput(title) + "' " +
				standard_book_group;
			return ProcessQuery(sqlStr);
		}

		public DataSet SearchForBookByAuthor(string author) 
		{			
			string sqlStr = standard_book_select + 
				"from books,publishers,book_cat,authors,book_inv,discounts " +
				"where publisher=publishers.id " +
				"and discounts.id=discount " +
				"and book_cat.id=category " +
				"and authors.bookid=isbn " +
				"and book_inv.bookid=isbn " +
				"and authors.name like '" + FormatInput(author) + "' " +
				standard_book_group;
			return ProcessQuery(sqlStr);
		}

		public DataSet SearchForBookByKeyword(string keyword) 
		{			
			string sqlStr = standard_book_select + 
				"from books,publishers,book_cat,keywords,book_inv,discounts " +
				"where publisher=publishers.id " +
				"and discounts.id=discount " +
				"and book_cat.id=category " +
				"and keywords.bookid = isbn " +
				"and book_inv.bookid=isbn " +
				"and keywords.word like '" + FormatInput(keyword) + "' " +
				standard_book_group;
			return ProcessQuery(sqlStr);
		}

		public DataSet SearchForBookByISBN(Int64 isbn) 
		{			
			string sqlStr = standard_book_select + 
				"from books,publishers,book_cat,book_inv,discounts " +
				"where publisher=publishers.id " +
				"and discounts.id=discount " +
				"and book_cat.id=category " +
				"and book_inv.bookid=isbn " +
				"and isbn=" + isbn.ToString() + " " +
				standard_book_group;
			return ProcessQuery(sqlStr);
		}

		public DataSet SearchForBookAuthors(Int64 isbn) 
		{
			string sqlStr = "select distinct authors.name " +
				"from books,authors " +
				"where authors.bookid=isbn " +
				"and isbn = " + isbn.ToString();			
			return ProcessQuery(sqlStr);
		}

		public DataSet SearchForAllBooks() 
		{
			string sqlStr = standard_book_select + 
				"from books,publishers,book_cat,book_inv,discounts " +
				"where publisher=publishers.id " +
				"and discounts.id=discount " +
				"and book_cat.id=category " + 
				"and book_inv.bookid=isbn " +
				standard_book_group;
			return ProcessQuery(sqlStr);
		}

		public DataSet SearchForMusicByTitle(string title) 
		{			
			string sqlStr = standard_music_select + 
				"from music,record_companies,music_genre,music_inv,discounts " +
				"where recordco=record_companies.id " +
				"and discounts.id=discount " +
				"and music_genre.id=genre " +
				"and music_inv.musicid=cdid " +
				"and title like '" + FormatInput(title) + "' " +
				standard_music_group;
			return ProcessQuery(sqlStr);
		}


		public DataSet SearchForMusicByArtist(string artist) 
		{
			string sqlStr = standard_music_select + 
				"from music,record_companies,music_genre,artists,music_inv,discounts " +
				"where recordco=record_companies.id " +
				"and discounts.id=discount " +
				"and music_genre.id=genre " +
				"and artists.musicid=cdid " +
				"and music_inv.musicid=cdid " +
				"and artists.name like '" + FormatInput(artist) + "' " +
				standard_music_group;
			return ProcessQuery(sqlStr);
		}

		public DataSet SearchForMusicByCDID(Int64 cdid) 
		{
			string sqlStr = standard_music_select + 
				"from music,record_companies,music_genre,music_inv,discounts " +
				"where recordco=record_companies.id " +
				"and discounts.id=discount " +
				"and music_genre.id=genre " +
				"and music_inv.musicid=cdid " +
				"and cdid=" + cdid.ToString() + " " +
				standard_music_group;
			return ProcessQuery(sqlStr);
		}
		
		public DataSet SearchForMusicArtists(Int64 cdid) 
		{
			string sqlStr = "select distinct artists.name " +
				"from music,artists " +
				"where artists.musicid=cdid " +
				"and cdid = " + cdid.ToString();
			return ProcessQuery(sqlStr);
		}

		public DataSet SearchForAllMusic() 
		{
			string sqlStr = standard_music_select + 
				"from music,record_companies,music_genre,music_inv,discounts " +
				"where recordco=record_companies.id " +
				"and discounts.id=discount " +
				"and music_genre.id=genre " +
				"and music_inv.musicid=cdid " +
				standard_music_group;
			return ProcessQuery(sqlStr);
		}

		private DataSet ProcessQuery(string query) 
		{
			SQLRequest request = new SQLRequest(this.server, query);          
			return request.Invoke();
		}

		private String FormatInput(string input) 
		{
			Regex r = new Regex("'");
			return "%" + r.Replace(input,"''") + "%";
		}
	}
}
