using System;
using System.Collections;
using System.Data;

namespace foofaraw.inventory
{
	public class QueryResult
	{
		public ArrayList books;
		public ArrayList music;
		public int increment_size;
		public int footer_display;

		public QueryResult(DataSet b, DataSet m)
		{
			ParseBooks(b);
			ParseMusic(m);
			increment_size = 10;
			footer_display = 10;
		}

		private void ParseBooks(DataSet book_data) 
		{
			books = new ArrayList();
			if(book_data == null)
				return;
			DataTable results = book_data.Tables[0];
			foreach(DataRow row in results.Rows) 
			{
				this.books.Add(new Book(Convert.ToInt64(row["isbn"]),
					(string)row["title"],
					GetAuthors(Convert.ToInt64(row["isbn"])),
					(string)row["publisher"],
					(DateTime)row["date"],
					(Decimal)row["price"],
					(string)row["category"],
					(int)row["availible"],
					(string)row["disc_type"],
					(DateTime)row["disc_expire"],
					Convert.ToInt16(row["disc_percent"])));    
			}
		}

		private void ParseMusic(DataSet music_data)
		{
			music = new ArrayList();
			if(music_data == null)
				return;
			DataTable results = music_data.Tables[0];
			foreach(DataRow row in results.Rows) 
			{
				this.music.Add(new Music(Convert.ToInt64(row["cdid"]),
					(string)row["title"],
					GetArtists(Convert.ToInt64(row["cdid"])),
					(string)row["recordco"],
					(DateTime)row["date"],
					(Decimal)row["price"],
					(string)row["genre"],
					(int)row["availible"],
					(string)row["disc_type"],
					(DateTime)row["disc_expire"],
					Convert.ToInt16(row["disc_percent"])));     
			}
		}

		private ArrayList GetAuthors(Int64 isbn) 
		{
			InventorySearch search = new InventorySearch();
			DataTable results = search.SearchForBookAuthors(isbn).Tables[0];
			ArrayList authors = new ArrayList();
			foreach(DataRow row in results.Rows)
				authors.Add((string)row["name"]);
			return authors;
		}

		private ArrayList GetArtists(Int64 cdid) 
		{
			InventorySearch search = new InventorySearch();
			DataTable results = search.SearchForMusicArtists(cdid).Tables[0];
			ArrayList artists = new ArrayList();
			foreach(DataRow row in results.Rows)
				artists.Add((string)row["name"]);
			return artists;
		}


		public string CreateBookResults(int increments, string type, string search) 
		{			
			string html = "<div class=\"searchheader\">Books</div>";
			if(this.books.Count == 0)
				return html + NoResults();
			if(this.increment_size*(increments-1)>=this.books.Count)
				return string.Empty;
			int start = GetStart(this.books,increments);
			for(int i=start; i<books.Count && i<(start+this.increment_size) &&
				this.increment_size*(increments-1)<this.books.Count; i++) 
			{
				Book b = (Book)this.books[i];
				SearchResultTemplate template = new SearchResultTemplate();	
				template.InsertItemLeft("Title",b.title);
				template.InsertItemLeft("Author(s)",b.authors);
				template.InsertItemLeft("Price","$"+Decimal.Round(b.price,2).ToString());
				template.InsertItemLeft("Discount",b.disc_type);
				template.InsertItemRight("Year",b.date.Year.ToString());
				template.InsertItemRight("Publisher",b.publisher);
				template.InsertItemRight("ISBN",b.isbn.ToString());
				template.InsertItemRight("Availible",b.availible.ToString());
				html += template.create(b.isbn.ToString(),"book",createpage(type,search,increments)) + "\n";
			}      		
			return html + GetFooter(increments,type,search,books);
		}

		public string CreateMusicResults(int increments, string type, string search) 
		{
			string html = "<div class=\"searchheader\">Music</div>";
			if(this.music.Count == 0)
				return html + NoResults();
			if(this.increment_size*(increments-1)>=this.music.Count)
				return string.Empty;
			int start = GetStart(this.music,increments);
			for(int i=start; i<this.music.Count && i<(start+this.increment_size); i++) 
			{
				Music m = (Music)this.music[i];
				SearchResultTemplate template = new SearchResultTemplate();	
				template.InsertItemLeft("Title",m.title);
				template.InsertItemLeft("Artist(s)",m.artists);
				template.InsertItemLeft("Price","$"+Decimal.Round(m.price,2).ToString());
				template.InsertItemLeft("Discount",m.disc_type);
				template.InsertItemRight("Year",m.date.Year.ToString());
				template.InsertItemRight("Record Company",m.recordco);
				template.InsertItemRight("CDID",m.cdid.ToString());
				template.InsertItemRight("Availible",m.availible.ToString());
				html += template.create(m.cdid.ToString(),"music",createpage(type,search,increments)) + "\n";
			}      		
			return html + GetFooter(increments,type,search,music);
		}

		private string createpage(string type,string search,int count) 
		{
			return "productsearch.aspx%3F"+type+"%3D"+search+"%26count%3D"+count.ToString();
		}

		private string NoResults()
		{
			return "Search returned no results";
		}

		private int GetStart(ArrayList items, int increments) 
		{
			if(increments <= 0)
				return 0;
			else if(items.Count < (increments-1) * this.increment_size)
				return 0;
			else
				return (increments-1) * this.increment_size;
		}

		public string GetFooter(int increments, string type, string search, ArrayList items) 
		{
			int upper;
			if(increments*this.increment_size<items.Count)
				upper = increments*this.increment_size;
			else
				upper = items.Count;
			string html = "<div class=\"searchfooter\"> ";
			html += "Viewing " + GetStart(items,increments) + " to " +
				upper + " of " + items.Count + " results.<br />";
			if(increments > 1 && GetStart(items,increments)!=0)
				html += CreateLink(increments-1,type,search," &lt; Previous ");
			if(increments * this.increment_size < items.Count)
				html += CreateLink(increments+1,type,search," Next &gt; ");
			return html + " </div>";
		}

		private string CreateLink(int increment, string type, string search, string text) 
		{
			return "<a href=\"productsearch.aspx?" + type + "=" + search + 
				"&count=" + increment +"\">" + text + "</a>";
		}
	}
}
