using System;
using System.Data;
using foofaraw.inventory;

namespace foofaraw.transactions
{
	public class CartAdder
	{
		public void AddToCart(string type, string id, ShoppingCart cart) 
		{
			if(id != null && type != null && cart != null) 
			{
				Int64 id_number;
				try 
				{
					id_number = Int64.Parse(id);
				}
				catch 
				{
					return;
				}

				if(type=="book") 
				{		
					AddBook(id_number,cart);
				}
				else if(type=="music") 
				{
					AddMusic(id_number,cart);
				}
			}
		}

		public void AddBook(Int64 id, ShoppingCart cart) 
		{
			BookCartItem book = GetBook(id);
			if(book != null)
				cart.AddBook(book);
		}

		public void AddMusic(Int64 id, ShoppingCart cart) 
		{
			MusicCartItem music = GetMusic(id);
			if(music != null)
				cart.AddMusic(music);
		}

		private MusicCartItem GetMusic(Int64 id)
		{
			InventorySearch search = new InventorySearch();

			DataTable table = search.SearchForMusicByCDID(id).Tables[0];
			DataTable artists = search.SearchForMusicArtists(id).Tables[0];
			
            if(table.Rows.Count==0)
				return null;
			else 
			{
                DataRow row = table.Rows[0];

				string artist = artists.Rows[0]["name"].ToString();
			                
                MusicCartItem mci = new MusicCartItem(artist,row["title"].ToString(),
					                                  row["cdid"].ToString(),(Decimal)row["price"],1,
					                                  Convert.ToDouble(row["weight"]), 0);

                mci.companyName = row["company"].ToString();

                return mci;
			}
		}

		private BookCartItem GetBook(Int64 id) 
		{
			InventorySearch search = new InventorySearch();

			DataTable table = search.SearchForBookByISBN(id).Tables[0];
			DataTable authors = search.SearchForBookAuthors(id).Tables[0];
			
            if(table.Rows.Count==0)
				return null;
			else 
			{
                DataRow row = table.Rows[0];
				string author = authors.Rows[0]["name"].ToString();
				
				BookCartItem bci = new BookCartItem(row["title"].ToString(),author,
					                                row["isbn"].ToString(),(Decimal)row["price"],1,
					                                Convert.ToDouble(row["weight"]), 0);

                bci.companyName = row["company"].ToString();

                return bci;
			}
		}
	}
}
