using System;
using System.Collections;

namespace foofaraw.transactions
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	[Serializable]
	public class CartItem
	{
		string _ItemID;
		decimal _Price;
		int _Quantity;
		double _SingleItemWeight;
		int _discountPercentage;
		string _companyName;

		#region Getters and Setters
		
        public string companyName
		{
			get { return _companyName; }
			set { _companyName = value; }
		}
		
		public string ItemID
		{
			get { return _ItemID; }
			set { _ItemID = value; }
		}

		public decimal Price
		{
			get {  return _Price*((100-_discountPercentage)/100); }
			set { _Price = value; }
		}

		public string Price_twodec
		{
			get { return String.Format("{0:c}", _Price*((100-_discountPercentage)/100)); }
		}


		public int Quantity
		{
			get { return _Quantity; }
			set { _Quantity = value; }
		}

        public int discountPercentage
		{
			get { return _discountPercentage; }
			set { _discountPercentage = value; }
		}
		
        public double SingleItemWeight
		{
			get { return _SingleItemWeight; }
			set { _SingleItemWeight = value; }
		}
		
        #endregion

		public CartItem (string ItemID, decimal Price, int Quantity, double SingleItemWeight, int discountPercentage)
		{
			_ItemID = ItemID;
			_Price = Price;
			_Quantity = Quantity;
			_SingleItemWeight = SingleItemWeight;
			_discountPercentage = discountPercentage;
			_companyName = "";
		}
		
        public CartItem (string ItemID, decimal Price, int Quantity, double SingleItemWeight, int discountPercentage, string companyName)
        {
            _ItemID = ItemID;
            _Price = Price;
            _Quantity = Quantity;
            _SingleItemWeight = SingleItemWeight;
            _discountPercentage = discountPercentage;
            _companyName = companyName;
        }		

	}


    public class MusicCartItem : CartItem
	{
		string _Artist;
		string _Album;

		#region Getters and Setters
		public string Artist
		{
			get { return _Artist; }
			set { _Artist = Artist; }
		}

		public string Album
		{
			get { return _Album; }
			set { _Album = Album; }
		}

		#endregion

		public MusicCartItem(string Artist, string Album, string ItemID,  decimal Price, int Quantity, double SingleItemWeight, int discountPercentage) : base(ItemID, Price, Quantity, SingleItemWeight, discountPercentage)
		{
			_Artist = Artist;
			_Album = Album;
		}

		public MusicCartItem(MusicCartItem c) : base(c.ItemID, c.Price, c.Quantity, c.SingleItemWeight, c.discountPercentage, c.companyName)			
		{
			_Artist = c.Artist;
			_Album = c.Album;
		}
	}

	
    public class BookCartItem : CartItem
	{
		string _Title;
		string _Author;

		#region Getters and Setters
	
        public string Title
		{
			get { return _Title; }
			set { _Title = Title; }
		}

		public string Author
		{
			get { return _Author; }
			set { _Author = Author; }
		}

		#endregion

		public BookCartItem(string Title, string Author, string ItemID,  decimal Price, int Quantity, double SingleItemWeight, int discountPercentage) : base(ItemID, Price, Quantity, SingleItemWeight, discountPercentage)
		{
			_Title = Title;
			_Author = Author;
		}

		public BookCartItem(BookCartItem b) : base(b.ItemID, b.Price, b.Quantity, b.SingleItemWeight, b.discountPercentage, b.companyName)
		{
			_Title = b.Title;
			_Author = b.Author;            
		}
	}


	
	[Serializable]
	public class ShoppingCart
	{
		Hashtable _MusicOrders = new Hashtable();
		Hashtable _BookOrders = new Hashtable();

		public bool isEmpty()
		{
			return ((_MusicOrders.Count == 0) && (_BookOrders.Count == 0));
		}
		
        public Hashtable MusicItems
        {
            get
            {
                return this._MusicOrders;
            }
        }

        public Hashtable BookItems
        {
            get
            {
                return this._BookOrders;
            }
        }

        public ICollection MusicOrders
        {
            get { return _MusicOrders.Values; }
        }

		public ICollection BookOrders
		{
			get { return _BookOrders.Values; }
		}

		public decimal MusicOrderTotalCost
		{
			get 
			{
				decimal total = 0;
				foreach (DictionaryEntry entry in _MusicOrders)
				{
					MusicCartItem order = (MusicCartItem) entry.Value;
					total += (order.Price * order.Quantity);
				}
				return total;
			}
		}

		public decimal BookOrderTotalCost
		{
			get 
			{
				decimal total = 0;
				foreach (DictionaryEntry entry in _BookOrders)
				{
					BookCartItem order = (BookCartItem) entry.Value;
					total += (order.Price * order.Quantity);
				}
				return total;
			}
		}


		public decimal TotalCost
		{
			get
			{
				return MusicOrderTotalCost + BookOrderTotalCost;
			}
		}

		public void AddMusic(MusicCartItem order)
		{
			MusicCartItem itemInCart = (MusicCartItem) _MusicOrders[order.ItemID];
			if (itemInCart != null)
				itemInCart.Quantity += order.Quantity;
			else
				_MusicOrders.Add (order.ItemID, order);
		}

		public void AddBook(BookCartItem order)
		{
			BookCartItem itemInCart = (BookCartItem) _BookOrders[order.ItemID];
			if (itemInCart != null)
				itemInCart.Quantity += order.Quantity;
			else
				_BookOrders.Add (order.ItemID, order);
		}


		public void RemoveBookOrder (string ItemID)
		{
			if (_BookOrders[ItemID] != null)
				_BookOrders.Remove (ItemID);
		}

		public void RemoveMusicOrder (string ItemID)
		{
			if (_MusicOrders[ItemID] != null)
				_MusicOrders.Remove (ItemID);
		}

		public void SetBookOrderQuantity (string ItemID, int quantity)
		{
			if(_BookOrders[ItemID] != null)
				((BookCartItem)_BookOrders[ItemID]).Quantity = quantity;
		}

		public void SetMusicOrderQuantity (string ItemID, int quantity)
		{
			if(_MusicOrders[ItemID] != null)
				((MusicCartItem)_MusicOrders[ItemID]).Quantity = quantity;
		}


		public ShoppingCart()
		{
			//
			// TODO: Add constructor logic here
			//
		}
	}


}
