using System;
using System.Collections;
using SqlRequest;
using System.Data;
using System.Xml;

namespace foofaraw.transactions
{
	public class ShippingOption
	{
		int _methodID;
		string _methodName;
		string _vendorName;
		decimal _price;
		double _deliveryTimePeriod;

		public ShippingOption(int methodID, string methodName, string vendorName, decimal price, double deliveryTimePeriod)
		{
			_methodID = methodID;
			_methodName = methodName;
			_vendorName = vendorName;
			_price = price;
			_deliveryTimePeriod = deliveryTimePeriod;

		}
		
		public string optionDisplay
		{
			get
			{
				return vendorName + ": " + methodName + "(Exp. Arrival: " + DateTime.Today.AddDays(deliveryTimePeriod).ToString() + ") - " + String.Format("{0:c}", price);
			}
		}

		public int methodID
		{
			get{ return _methodID; }
			set{ _methodID = value; }
		}
		public string methodName
		{
			get{ return _methodName; }
			set{ _methodName = value; }
		}
		public string vendorName
		{
			get{ return _vendorName; }
			set{ _vendorName = value; }
		}
		public decimal price
		{
			get{ return _price; }
			set{ _price = value; }
		}
		public double deliveryTimePeriod
		{
			get{ return _deliveryTimePeriod; }
			set{ _deliveryTimePeriod = value; }
		}
		public string expectedDeliveryDate
		{
			get
			{
				return (DateTime.Today.AddDays(_deliveryTimePeriod)).ToString();
			}
		}
		
	}



	public class Package
	{
		Hashtable _musicList;	//MusicCartItem Objects
		Hashtable _bookList;	//BookCartItem Objects
		string _warehouseID;
		string _companyName;
		Hashtable _shippingOptions;
		ShippingOption _chosenOption;

		public Package()
		{
			_musicList = new Hashtable();
			_bookList = new Hashtable();
			_shippingOptions = new Hashtable();
			_companyName = "";
		}

		public void populateShippingOptions(string city)
		{
			if(this.companyName == "")
			{
				TransactionQueries query = new TransactionQueries();
				DataSet possibleOptions = query.getShippingOptionsList();
				foreach(DataRow row in possibleOptions.Tables[0].Rows)
				{
					string optionName = row["method_name"].ToString();
					DataSet cheapestOption = query.getCheapestOption(optionName, city, warehouseID, pkgWeight, numItems);
					int numReturned = cheapestOption.Tables[0].Rows.Count;
					if(numReturned > 0)
					{
						DataRow cheapRow = cheapestOption.Tables[0].Rows[0];
						ShippingOption op = new ShippingOption(Convert.ToInt32(cheapRow["method_id"].ToString()), cheapRow["method_name"].ToString(), 
                            cheapRow["vendor_name"].ToString(), Convert.ToDecimal(cheapRow["ship_price"].ToString()), Convert.ToDouble(cheapRow["delivery_time"].ToString()));
                        
                        _shippingOptions.Add(op.methodID, op);
					}
				
				}
			} 
			else 
			{
				TransactionQueries query = new TransactionQueries();
				DataSet possibleOptions = query.getShippingOptionsList(companyName);
				foreach(DataRow row in possibleOptions.Tables[0].Rows)
				{
					string optionName = row["method_name"].ToString();
					DataSet cheapestOption = query.getOtherCompCheapestOption(optionName, city, warehouseID, pkgWeight, numItems, this.companyName);
					int numReturned = cheapestOption.Tables[0].Rows.Count;
					if(numReturned > 0)
					{
						DataRow cheapRow = cheapestOption.Tables[0].Rows[0];
						ShippingOption op = new ShippingOption(Convert.ToInt32(cheapRow["method_id"].ToString()), cheapRow["method_name"].ToString(), cheapRow
							["vendor_name"].ToString(), Convert.ToDecimal(cheapRow["ship_price"].ToString()), Convert.ToDouble(cheapRow["delivery_time"].ToString()));
						_shippingOptions.Add(op.methodID, op);
					}
				
				}

			}

		}

		#region Trivial Getters and Setters for Shipping and Billing Data
		
		public int chosenOptionMethodID
		{
			get 
			{
				if(_chosenOption != null)
					return _chosenOption.methodID;
				else 
					return 1;
			}
		}


			public ShippingOption chosenOption
		{
			get { return _chosenOption; }
			set { _chosenOption = value; }
		}
		
			public Hashtable musicList
		{
			get { return _musicList; }
			set { _musicList = value; }
		}

		public Hashtable bookList
		{
			get { return _bookList; }
			set { _bookList = value; }
		}

		public Hashtable shippingOptionsHash
		{
			get { return _shippingOptions; }
			
		}

		public ICollection shippingOptions
		{
			get { return _shippingOptions.Values; }
			
		}

		public string warehouseID
		{
			get { return _warehouseID; }
			set { _warehouseID = value; }
		}

		public string companyName
		{
			get {return _companyName;}
			set {_companyName = value; }
		}

		#endregion
		
		public int numItems
		{
			get
			{
				int cnt = 0;

                foreach (DictionaryEntry entry in _musicList)
				{
					MusicCartItem order = (MusicCartItem) entry.Value;
					cnt += (order.Quantity);
				}

				foreach (DictionaryEntry entry in _bookList)
				{
					BookCartItem order = (BookCartItem) entry.Value;
					cnt += (order.Quantity);
				}
				return cnt;
			}
		
		}

		//still need to implement this
		public double pkgWeight
		{
			get
			{
				double weight = 0;
				foreach (DictionaryEntry entry in _musicList)
				{
					MusicCartItem order = (MusicCartItem) entry.Value;
					weight += (order.SingleItemWeight * order.Quantity);
				}
				foreach (DictionaryEntry entry in _bookList)
				{
					BookCartItem order = (BookCartItem) entry.Value;
					weight += (order.SingleItemWeight * order.Quantity);
				}
				return weight;
			}
		}
	}

  
	/// <summary>
	/// The Transaction object stores all of the transaction information while the customer is going through the 
	/// checkout process.  It is stored in a session object.
	/// Here is a sample list of canned packages:	
	/// </summary>
	public class Transaction
	{
		
		#region Internal shiznit

		internal class PersonalInfo
		{
			internal string fName;
			internal string lName;
			internal string address;
			internal string city;
			internal string stateID;
			internal string zip;
			internal string postalCode;
			internal string phone;
			internal string eMail;
		}
		
     
		internal enum Mode {BOOKS, MUSIC};

		#endregion

		#region private Data
		
		private string _specialInstructions;
		private ShoppingCart cart;		//Shopping Cart that we are doing a transaction with ;)
		private ArrayList _packageList;
		private Int64 _invoiceNumber;
		private string _paymentType;
		private string _nameOnCard;
		private string _cardNum;
		private string _expDate;

		PersonalInfo buyer;
		PersonalInfo recipient;
		
		#endregion

		public void commitTransaction(string loginName, string dir)
		{
			this._invoiceNumber = getInvoiceNumber();
			

			CommitBillingInfo(loginName);
			CommitShippingInfo();
			CommitInventoryInfo();

            foreach(Package p in this.packageList)
            {
                if(p.companyName != String.Empty)
                {                   
                   XMLCreator orderCreator = new XMLCreator();
                   
                   orderCreator.CreateOrderXML(p, this);

                   orderCreator.SaveDocument(dir + p.companyName + this._invoiceNumber.ToString() + ".xml");

                   orderCreator.ClearDocument();                   
                }
            }

		}

		#region Public

        public decimal costShipping
		{
			get 
			{ 
				decimal sCost = 0;
				if(_packageList.Count > 0)
				{
					for(int i = 0; i < _packageList.Count; i++)
					{
						Package p =  ((Package)_packageList[i]);
						if(p.chosenOption !=null)
							sCost += p.chosenOption.price;
					}
				}
				return Convert.ToDecimal(sCost);
				
			}
		}

		
        public decimal costTax
		{
			get
			{
				return subTotal*Convert.ToDecimal(.081);}
		}


        public decimal subTotal
		{
			get{ return cart.TotalCost;}
		}
		
  
        public decimal TotalCost
		{
			get
			{
				return costShipping + costTax + subTotal;
			}
		}

	
        #endregion

		#region Getters and Setters for Shipping and Billing Data
        
		public string paymentType
		{
			get { return _paymentType; }
			set  { _paymentType = value; }
		}
		
		
		public string nameOnCard
		{
			get { return _nameOnCard; }
			set  { _nameOnCard = value; }
		}
		
		
		public string cardNum
		{
			get { return _cardNum; }
			set  { _cardNum = value; }
		}
		
		
		public string expDate
		{
			get { return _expDate; }
			set  { _expDate = value; }
		}
		
	
		public string specialInstructions
		{
			get { return _specialInstructions; }
			set  { _specialInstructions = value; }
		}

		
		public ArrayList packageList
		{
			get { return _packageList; }
			set { _packageList = value; }
		}

		
		public string bFName
		{
			get { return buyer.fName; }
			set { buyer.fName = value; }
		}

		
		public string bLName
		{
			get { return buyer.lName; }
			set { buyer.lName = value; }
		}

		
		public string bAddress
		{
			get { return buyer.address; }
			set { buyer.address = value; }
		}

		
		public string bCity
		{
			get { return buyer.city; }
			set { buyer.city = value; }
		}

		
		public string bStateID
		{
			get { return buyer.stateID; }
			set { buyer.stateID = value; }
		}
   
		
		public string bZip
		{
			get { return buyer.zip; }
			set { buyer.zip = value; }
		}

		
		public string bPostalCode
		{
			get { return buyer.postalCode; }
			set { buyer.postalCode = value; }
		}

		
		public string bPhone
		{
			get { return buyer.phone; }
			set { buyer.phone = value; }
		}

		
		public string bEmail
		{
			get { return buyer.eMail; }
			set { buyer.eMail = value; }
		}

		
		public string sFName
		{
			get 
			{ 
				if(this.recipient == null)
					return this.buyer.fName;
				else
					return this.recipient.fName;
                
			}
			set 
			{
				if(this.recipient == null)
					this.buyer.fName = value;
				else
					this.recipient.fName = value;
			}
		}


		public string sLName
		{
			get 
			{
				if(this.recipient == null)
					return this.buyer.lName;
				else
					return this.recipient.lName;
			}
			set 
			{ 
				if(this.recipient == null)
					this.buyer.lName = value;
				else
					this.recipient.lName = value;
			}
		}

		
		public string sAddress
		{
			get 
			{ 
				if(this.recipient == null)
					return this.buyer.address;
				else
					return this.recipient.address;
			}
			set 
			{ 
				if(this.recipient == null)
					this.buyer.address = value;
				else
					this.recipient.address = value;
			}
		}

		
		public string sCity
		{
			get 
			{ 
				if(this.recipient == null)
					return this.buyer.city;
				else
					return this.recipient.city;
			}
			set 
			{ 
				if(this.recipient == null)
					this.buyer.city = value;
				else
					this.recipient.city = value;
			}
		}

		
		public string sStateID
		{
			get 
			{ 
				if(this.recipient == null)
					return this.buyer.stateID;
				else
					return this.recipient.stateID;
			}

			set 
			{
				if(this.recipient == null)
					this.buyer.stateID = value;
				else
					this.recipient.stateID = value;
			}
		}

		
		public string sZip
		{
			get 
			{ 
				if(this.recipient == null)
					return this.buyer.zip;
				else
					return this.recipient.zip;
			}

			set 
			{
				if(this.recipient == null)
					this.buyer.zip = value;
				else
					this.recipient.zip = value;
			}
		}

		
		public string sPostalCode
		{
			get 
			{ 
				if(this.recipient == null)
					return this.buyer.postalCode;
				else
					return this.recipient.postalCode;
			}

			set 
			{ 
				if(this.recipient == null)
					this.buyer.postalCode = value;
				else 
					this.recipient.postalCode = value;
			}
		}

		
		public string sPhone
		{
			get 
			{
				if(this.recipient == null)
					return this.buyer.phone;
				else
					return this.recipient.phone;
			}
			set 
			{ 
				if(this.recipient == null)
					this.buyer.phone = value;
				else
					this.recipient.phone = value;
			}
		}

		
		public string sEmail
		{
			get 
			{ 
				if(this.recipient == null)
					return this.buyer.eMail;
				else
					return this.recipient.eMail;
			}

			set 
			{ 
				if(this.recipient == null)
					this.buyer.eMail = value;
				else 
					this.recipient.eMail = value;
			}
		}
		
	
		#endregion 

		#region Constructor

		public Transaction(ShoppingCart cart)
		{
			this.cart = cart;
			_packageList = new ArrayList();
			buyer = new PersonalInfo();
			recipient = new PersonalInfo();
		}


		#endregion

		#region Private Methods
       
		private Int64 getInvoiceNumber()
		{
			string inv_number = "SELECT Max(InvoiceNumber) as InvoiceNumber from invoice";
			SqlRequest.SQLRequest i_numb = new SqlRequest.SQLRequest(SqlRequest.StaticServer.RyanM, inv_number);
			Int64 InvoiceNumber;

			DataSet temp = i_numb.Invoke();

			if(temp.Tables[0].Rows[0]["InvoiceNumber"] is DBNull)
				InvoiceNumber = 0;
			else
				InvoiceNumber = Convert.ToInt64(temp.Tables[0].Rows[0]["InvoiceNumber"]) + 1;

			return InvoiceNumber;
		}

		
		private string getQuery(Hashtable Items, Mode m)
		{
			string SqlQuery = "select *\nfrom ";
			string columnName = (m == Mode.BOOKS) ? "bookid" : "musicid";

			if(Items.Count == 0)
				return String.Empty;

			if(m == Mode.BOOKS)
			{
				SqlQuery += "book_inv\nwhere ";
			}
			else
			{
				SqlQuery += "music_inv\nwhere ";
			}
			SqlQuery += columnName + " = ";

			IDictionaryEnumerator e = Items.GetEnumerator();
			while(e.MoveNext())
				SqlQuery += e.Key.ToString() + " OR " + columnName + " = ";
            
			SqlQuery = SqlQuery.Remove(SqlQuery.LastIndexOf(" OR"), SqlQuery.Length - SqlQuery.LastIndexOf(" OR"));
			SqlQuery += "\norder by location";

			return SqlQuery;
		}


		public void DetermineWarehouses()
		{           
			SqlRequest.SQLServer s1 = new SqlRequest.SQLServer("IISQLSRV.cs.washington.edu", "chuggins", "moocow", "chuggins_inventory");
			SqlRequest.SQLServer s2 = new SqlRequest.SQLServer("IISQLSRV.cs.washington.edu", "chuggins", "moocow", "chuggins_inventory");
			SqlRequest.SQLServer s3 = new SqlRequest.SQLServer("IISQLSRV.cs.washington.edu", "chuggins", "moocow", "chuggins_inventory");           
			
			string BookString = this.getQuery(this.cart.BookItems, Mode.BOOKS);
			string MusicString = this.getQuery(this.cart.MusicItems, Mode.MUSIC);
                     
			SqlRequest.SQLRequest req1 = new SqlRequest.SQLRequest(s1, "select * from wharehouses", "wharehouses");
			SqlRequest.SQLRequest req2 = new SqlRequest.SQLRequest(s2, BookString, "book_inv");
			SqlRequest.SQLRequest req3 = new SqlRequest.SQLRequest(s3, MusicString, "music_inv");

			SqlRequest.SQLBatch b;
			
			if(BookString == String.Empty)
			{
				b = new SqlRequest.SQLBatch(req1, req3);

				b.SetRelations(new SqlRequest.Relation("music", "wharehouses", "id", "music_inv", "location"));
			}
			else if(MusicString == String.Empty)
			{
				b = new SqlRequest.SQLBatch(req1, req2);

				b.SetRelations(new SqlRequest.Relation("books", "wharehouses", "id", "book_inv", "location"));
			}
			else
			{
				b = new SqlRequest.SQLBatch(req1, req2, req3);

				b.SetRelations(new SqlRequest.Relation("books", "wharehouses", "id", "book_inv", "location"),
							   new SqlRequest.Relation("music", "wharehouses", "id", "music_inv", "location"));
			}

			//go go power rangers
			b.RunQueries();
             			    
			Hashtable bookCopy = DeepCopyHashtable(this.cart.BookItems, Mode.BOOKS);
			Hashtable musicCopy = DeepCopyHashtable(this.cart.MusicItems, Mode.MUSIC);

			//try to determine if any of the items they want exist in a wharehouse in their city
			MatchCity(bookCopy, musicCopy, b.Results, this.sCity, this._packageList);
   
			if(bookCopy.Count != 0 || musicCopy.Count != 0)
				MatchItemsToWarehouses(bookCopy, musicCopy, this.sCity, b.Results, this._packageList);

            //the items from a foreign company will NOT be picked out by the above to algorithms
            //so we need to package them here
            if(bookCopy.Count != 0 || musicCopy.Count != 0)
            {
                ArrayList XMLOrders = new ArrayList();

                PackageForeignOrders(bookCopy, musicCopy, this._packageList);
            }
		}

        private void PackageForeignOrders(Hashtable books, Hashtable music, ArrayList packageList)
        {
            ArrayList sortedBooks = new ArrayList();
            ArrayList sortedMusic = new ArrayList();
            Hashtable packages = new Hashtable();

            //sort our books
            if(books.Count != 0)
            {
                foreach(DictionaryEntry d in books)
                    sortedBooks.Add(d.Value);

                sortedBooks.Sort(new BookComparer());
            }

            //sort our music
            if(music.Count != 0)
            {
                foreach(DictionaryEntry d in music)
                    sortedMusic.Add(d.Value);

                sortedMusic.Sort(new MusicComparer());
            }


            //package up our books
            if(books.Count != 0)
            {
                string curCompany = ((BookCartItem)sortedBooks[0]).companyName;
                Package cur = new Package();
                
                cur.companyName = curCompany;
                cur.warehouseID = "10";  //all foreign products are in this warehouse ;)

                for(int x = 0 ; x < sortedBooks.Count ; x++)
                {
                    BookCartItem bc = sortedBooks[x] as BookCartItem;

                    //new company, store the package and load up a new one
                    if(bc.companyName != curCompany)                                            
                    {
                        //keep this package around in case we need to add music to it
                        packages.Add(cur.companyName, cur);  

                        cur = new Package();

                        cur.companyName = bc.companyName;
                        cur.warehouseID = "10";                        
                    }

                    cur.bookList.Add(bc.ItemID, bc);
                }

                packages.Add(cur.companyName, cur);
            }

            if(music.Count != 0)
            {
                string curCompany = ((MusicCartItem)sortedMusic[0]).companyName;
                Package cur;
                                                           
                if(packages.ContainsKey(curCompany))
                    cur = packages[curCompany] as Package;
                else
                {
                    cur = new Package();

                    cur.companyName = curCompany;
                    cur.warehouseID = "10";  //all foreign products are in this warehouse ;)
                }
                    
                for(int x = 0 ; x < sortedMusic.Count ; x++)
                {
                    MusicCartItem mc = sortedMusic[x] as MusicCartItem;

                    //new company
                    if(mc.companyName != curCompany)
                    {                        
                        if(packages.ContainsKey(mc.companyName))
                            cur = packages[mc.companyName] as Package;
                        else
                        {
                            cur = new Package();
                            
                            cur.companyName = mc.companyName;
                            cur.warehouseID = "10";
                        }                        
                    }

                    cur.musicList.Add(mc.ItemID, mc);
                }            

                packages.Add(cur.companyName, cur);
            }

            foreach(DictionaryEntry d in packages)
                packageList.Add(d.Value);            
        }

     
        #region Internal Classes

        internal class BookComparer : IComparer
        {
            public int Compare(object a, object b)
            {
                BookCartItem a1 = a as BookCartItem;
                BookCartItem a2 = b as BookCartItem;

                return a1.companyName.CompareTo(a2.companyName);
            }
        }


        internal class MusicComparer : IComparer
        {
            public int Compare(object a, object b)
            {
                MusicCartItem a1 = a as MusicCartItem;
                MusicCartItem a2 = b as MusicCartItem;

                return a1.companyName.CompareTo(a2.companyName);
            }
        }


        #endregion

        private XmlDocument GenerateOrder(Package p)
        {

            return null;
        }

		
        private Hashtable DeepCopyHashtable(Hashtable toCopy, Mode m)
		{
			Hashtable toRet = new Hashtable();


			foreach(DictionaryEntry d in toCopy)
			{				
				if(m == Mode.BOOKS)
				{
					BookCartItem b = d.Value as BookCartItem;                    
                    
					toRet.Add(Convert.ToString(d.Key), new BookCartItem(b));
				}
				else
				{
					MusicCartItem mc = d.Value as MusicCartItem;

					toRet.Add(Convert.ToString(d.Key), new MusicCartItem(mc));
				}
			}

			return toRet;					
		}

		
        private void MatchItemsToWarehouses(Hashtable books, Hashtable music, string buyerCity, DataSet QueryResults, ArrayList Packages)
		{            
			foreach(DataRow dr in QueryResults.Tables["wharehouses"].Rows)
			{
				DataRow[] book_children = null;
				DataRow[] music_children = null;

				//if we have book info
				if(QueryResults.Relations.Contains("books"))
				{
					book_children = dr.GetChildRows("books");
				}

				if(QueryResults.Relations.Contains("music"))
				{
					music_children = dr.GetChildRows("music");
				}

				Package p = new Package();               
				p.warehouseID = Convert.ToString(dr["id"]);

                if(book_children != null)
                {
                    for(int i = 0 ; i < book_children.Length ; i++)
                    {                					    
                        string columnName = "bookid";
						string key = Convert.ToString(book_children[i][columnName]);
 
						//although the item is in the relation, as being in this warehouse
						//it could have been snagged by the first algo, ie from another warehouse
						//that is in the recipient's city
						if(!books.Contains(key))
							continue;
						
						if(dr["city"].ToString() == buyerCity)
							continue;

                        //don't touch this one here, it is a foreign order
                        if(((CartItem)books[key]).companyName != String.Empty)
                            continue;

						int QtyNeeded = Convert.ToInt32(((CartItem)books[key]).Quantity);
						int QtyOnHand = Convert.ToInt32(book_children[i]["availible"]);

                        if(QtyOnHand == 0)
                            continue;
                        
						//we can satisfy the request from here
						if(QtyOnHand >= QtyNeeded)
						{							
							CartItem c = books[key] as CartItem;

							//add to the booklist in this package
							p.bookList.Add(key, c);
                                
							books.Remove(key);
						}
						else
						{							
							BookCartItem current = books[key] as BookCartItem;                                								

							int newQtyNeeded = QtyNeeded - QtyOnHand;				                          

                            BookCartItem b = new BookCartItem(current.Title, current.Author, current.ItemID, current.Price, 
                                QtyOnHand, current.SingleItemWeight,
                                current.discountPercentage);

                            //add the book list item corresponding to what we took out of this warehouse
                            p.bookList.Add(key, b);

                            current.Quantity = newQtyNeeded;

						} 
					}
				}

				if(music_children != null)
				{
					for(int i = 0 ; i < music_children.Length ; i++)                     
					{
						string columnName = "musicid";
						string key = Convert.ToString(music_children[i][columnName]);
 
						//although the item is in the relation, as being in this warehouse
						//it could have been snagged by the first algo, ie from another warehouse
						//that is in the recipient's city
						if(!music.Contains(key))
							continue;
                        
                        if(dr["city"].ToString() == buyerCity)
                            continue;

                        if(((CartItem)music[key]).companyName != String.Empty)
                            continue;

						int QtyNeeded = Convert.ToInt32(((CartItem)music[key]).Quantity);
						int QtyOnHand = Convert.ToInt32(music_children[i]["availible"]);

                        if(QtyOnHand == 0)
                            continue;

						//we can satisfy the request from here
						if(QtyOnHand >= QtyNeeded)
						{							
							MusicCartItem c = music[key] as MusicCartItem;

							p.musicList.Add(key, c);									

							music.Remove(key);
						}
						else
						{							
							MusicCartItem current = music[key] as MusicCartItem;
                                               
							int newQtyNeeded = QtyNeeded - QtyOnHand;									
                                
							MusicCartItem b = new MusicCartItem(current.Artist, current.Album, current.ItemID, current.Price, QtyOnHand,
															    current.SingleItemWeight, current.discountPercentage);
                                                                                                            
							//add the book list item corresponding to what we took out of this warehouse
							p.musicList.Add(key, b);

							current.Quantity = newQtyNeeded;
						} 
					}                     
				}			

				if(p.bookList.Count != 0 || p.musicList.Count != 0)
					Packages.Add(p);
			}
		}


		private void MatchCity(Hashtable books, Hashtable music, DataSet results, string City, ArrayList Packages)
		{
			foreach(DataRow dr in results.Tables["wharehouses"].Rows)
			{
				Package p = new Package();

				//found a warehouse in their city
				if(dr["city"].ToString() == City)
				{					
					DataRow[] book_children = null;
					DataRow[] music_children = null;

					if(results.Relations.Contains("books"))
					{
						book_children = dr.GetChildRows("books");
					}

					if(results.Relations.Contains("music"))
					{
						music_children = dr.GetChildRows("music");
					}

					//nothing we want at this wharehouse
					if(book_children == null && music_children == null)
                        continue;
					
					p.warehouseID = Convert.ToString(dr["id"]);

                    if(book_children != null)
                    {
                        for(int i = 0 ; i < book_children.Length ; i++)                     
                        {
                            string columnName = "bookid";
                            string key = Convert.ToString(book_children[i][columnName]);

                            //don't touch this one here, it is a foreign order
                            if(((CartItem)books[key]).companyName != String.Empty)
                                continue;

							int QtyNeeded = Convert.ToInt32(((CartItem)books[key]).Quantity);
							int QtyOnHand = Convert.ToInt32(book_children[i]["availible"]);

                            if(QtyOnHand == 0)
                                continue;

                            //we can satisfy the request from here
                            if(QtyOnHand >= QtyNeeded)
                            {
								CartItem c = books[key] as CartItem;

                                //add to the booklist in this package
                                p.bookList.Add(key, c);
                                
                                books.Remove(key);
                            }
                            else
                            {
                                BookCartItem current = books[key] as BookCartItem;                                								

                                int newQtyNeeded = QtyNeeded - QtyOnHand;				
                                                             
                                BookCartItem b = new BookCartItem(current.Title, current.Author, current.ItemID, current.Price, 
                                                                  QtyOnHand, current.SingleItemWeight,
                                                                  current.discountPercentage);

                                //add the book list item corresponding to what we took out of this warehouse
                                p.bookList.Add(key, b);

                                current.Quantity = newQtyNeeded;
                            } 
                        }
                    }

                    if(music_children != null)
                    {
                        for(int i = 0 ; i < music_children.Length ; i++)                     
                        {
                            string columnName = "musicid";
                            string key = Convert.ToString(music_children[i][columnName]);
                            

                            //foreign item
                            if(((CartItem)music[key]).companyName != String.Empty)
                                continue;

                            int QtyNeeded = Convert.ToInt32(((CartItem)music[key]).Quantity);
							int QtyOnHand = Convert.ToInt32(music_children[i]["availible"]);

                            if(QtyOnHand == 0)
                                continue;

                            //we can satisfy the request from here
                            if(QtyOnHand >= QtyNeeded)
                            {
								MusicCartItem c = music[key] as MusicCartItem;

                                p.musicList.Add(key, c);									

                                music.Remove(key);
                            }
                            else
                            {
                                MusicCartItem current = music[key] as MusicCartItem;
                                               
                                int newQtyNeeded = QtyNeeded - QtyOnHand;									
                                
                                MusicCartItem b = new MusicCartItem(current.Artist, current.Album, current.ItemID, current.Price, QtyOnHand,
                                                                    current.SingleItemWeight, current.discountPercentage);
                                                                                                            
                                //add the book list item corresponding to what we took out of this warehouse
                                p.musicList.Add(key, b);

                                current.Quantity = newQtyNeeded;
                            } 
                        }                     
                    }
				}

				if(p.bookList.Count != 0 || p.musicList.Count != 0)
					Packages.Add(p);
			}			
		}
     
		
		private void CommitShippingInfo()
		{
			foreach (Package p in this.packageList)
			{

				string sql = "INSERT INTO Packages(invoice_id, weight, num_items, special_instructions, source_warehouse, order_status_id, date_shipped, expected_delivery_date, "+
					"shipping_method_id, shipping_price, shipping_Lname, shipping_Fname, shipping_address, "+
					"shipping_city, shipping_state, shipping_zip, shipping_phone) VALUES " +
					"("+this._invoiceNumber.ToString()+", "+p.pkgWeight+", "+p.numItems+", '"+this.specialInstructions+"', "+p.warehouseID+", 1, '"+DateTime.Today.ToString()+"', '"+p.chosenOption.expectedDeliveryDate+
					"', "+p.chosenOption.methodID+", "+p.chosenOption.price+", '"+this.sLName+"', '"+this.sFName+"', '"+this.sAddress+
					"', '"+this.sCity+"', 1, '"+this.sZip+"', '"+this.sPhone+"')";
				
				
				SqlRequest.SQLRequest i_up = new SqlRequest.SQLRequest(SqlRequest.StaticServer.TimP, sql);
				i_up.Invoke();
				//int i=1;
			}
		}

	
		private void CommitBillingInfo(string loginName)
		{			
			//invoice info
			string req = "INSERT INTO invoice(InvoiceNumber, OrderDate, Buyer, Recipient, BuyerIsRecipient)\n" +
					     "VALUES(" + this._invoiceNumber.ToString() + ", Cast('" + DateTime.Now.Month.ToString() + "/" + DateTime.Now.Day.ToString() + "/" + 
						 DateTime.Now.Year.ToString() + "' as DateTime), '" + this.bFName + " " + this.bLName + "', '" + this.sFName + " " + this.sLName + "', " +
					     ((this.recipient == null) ? 1 : 0) + ")";

			SqlRequest.SQLRequest i_up = new SqlRequest.SQLRequest(SqlRequest.StaticServer.RyanM, req);

			i_up.Invoke();

			//buyer info  are things like phone number and email required???
			if(this.bPostalCode != null)
			{
				req = "INSERT INTO buyer(InvoiceNumber, BuyerName, StreetAddress, City, State, ZipCode, PostalCode, PhoneNumber, EmailAddress)\n" +
					  "VALUES(" + this._invoiceNumber.ToString() + ", '" + this.bFName + " " + this.bLName + "', '" + this.bAddress + "', '" + this.bCity + "', '" +
					  this.bStateID + "', " + this.bZip + ", " + this.bPostalCode + ", " + this.bPhone + ", '" + this.bEmail + "')";
			}
			else
			{
				req = "INSERT INTO buyer(InvoiceNumber, BuyerName, StreetAddress, City, State, ZipCode, PhoneNumber, EmailAddress)\n" +
					"VALUES(" + this._invoiceNumber.ToString() + ", '" + this.bFName + " " + this.bLName + "', '" + this.bAddress + "', '" + this.bCity + "', '" +
					this.bStateID + "', " + this.bZip + ", " + this.bPhone + ", '" + this.bEmail + "')";
			}

			i_up = new SqlRequest.SQLRequest(SqlRequest.StaticServer.RyanM, req);

			i_up.Invoke();

			//if the buyer is the recipient then we don't duplicate the data
			if(this.recipient != null)
			{
				//buyer info  are things like phone number and email required???
				if(this.bPostalCode != null)
				{
					req = "INSERT INTO recipient(InvoiceNumber, RecipientName, StreetAddress, City, State, ZipCode, PostalCode, PhoneNumber, EmailAddress)\n" +
						"VALUES(" + this._invoiceNumber.ToString() + ", '" + this.sFName + " " + this.sLName + "', '" + this.sAddress + "', '" + this.sCity + "', '" +
						this.sStateID + "', " + this.sZip + ", " + this.sPostalCode + ", " + this.sPhone + ", " + this.sEmail + ")";

				}
				else
				{
					req = "INSERT INTO recipient(InvoiceNumber, RecipientName, StreetAddress, City, State, ZipCode, PhoneNumber, EmailAddress)\n" +
						"VALUES(" + this._invoiceNumber.ToString() + ", '" + this.sFName + " " + this.sLName + "', '" + this.sAddress + "', '" + this.sCity + "', '" +
						this.sStateID + "', " + this.sZip + ", " + this.sPhone + ", '" + this.sEmail + "')";
				}

				i_up = new SqlRequest.SQLRequest(SqlRequest.StaticServer.RyanM, req);

				i_up.Invoke();
				
			}

			IDictionaryEnumerator e;
			int Location = 0;
			foreach(Package p in this._packageList)
			{				
				if(p.companyName == "")
				{
					Location = int.Parse(p.warehouseID);

					ShippingOption so = p.chosenOption;
				
					e = p.bookList.GetEnumerator();

					while(e.MoveNext())
					{		
						BookCartItem b = e.Value as BookCartItem;

						req = "INSERT INTO items(InvoiceNumber, UniqueProductID, ItemName, Quantity, PricePerUnit, Location, EstimatedDeliveryDate, ItemType)\n" +
							"VALUES(" + this._invoiceNumber.ToString() + ", " + e.Key.ToString() + ", '" + b.Title + "', " + b.Quantity.ToString() +
							", " + b.Price.ToString() + ", " + Location.ToString() + ", Cast('" + so.expectedDeliveryDate + "' as DateTime), 0)";
					
						i_up = new SqlRequest.SQLRequest(SqlRequest.StaticServer.RyanM, req);

						i_up.Invoke();
					}			

					//add all music items to the items DB
					e = p.musicList.GetEnumerator();

					while(e.MoveNext())
					{
						MusicCartItem m = e.Value as MusicCartItem;
					
						req = "INSERT INTO items(InvoiceNumber, UniqueProductID, ItemName, Quantity, PricePerUnit, Location EstimatedDeliveryDate, ItemType)\n" +
							"VALUES(" + this._invoiceNumber.ToString() + ", " + e.Key.ToString() + ", '" + m.Album + "', " + m.Quantity.ToString() +
							", " +  m.Price.ToString() + ", " + Location.ToString() + ", Cast('" + so.expectedDeliveryDate + "' as DateTime), 1)";

						i_up = new SqlRequest.SQLRequest(SqlRequest.StaticServer.RyanM, req);

						i_up.Invoke();
					}
				}
			}
			
			req = "INSERT INTO login(Login, InvoiceNumber)\n" +
				  "VALUES('" + loginName + "', " + this._invoiceNumber.ToString() + ")";

			i_up = new SqlRequest.SQLRequest(SqlRequest.StaticServer.RyanM, req);

			i_up.Invoke();


			req = "INSERT INTO payment_type(InvoiceNumber, PaymentType, PaymentInfo, ExpirationDate, ExtraInfo)\n" +
				  "VALUES(" + this._invoiceNumber.ToString() + ", 0, " + this._cardNum + ", Cast('" + this._expDate + "' as DateTime), '" + 
				  loginName + "')";																  

			i_up = new SqlRequest.SQLRequest(SqlRequest.StaticServer.RyanM, req);

			i_up.Invoke();

		}

	
		private void CommitInventoryInfo()
		{
			string req;
			SqlRequest.SQLRequest i_up;

			foreach(Package p in this._packageList)
			{
				int warehouse = int.Parse(p.warehouseID);

				foreach(DictionaryEntry d in p.bookList)
				{
					int currentQty = GetCurrentQuantity(Convert.ToInt64(d.Key), warehouse, Mode.BOOKS);
					BookCartItem b = d.Value as BookCartItem;

					int NewQty = currentQty - b.Quantity;
					
					req = "UPDATE book_inv\nset availible = " + NewQty.ToString() + " where bookid = " + Convert.ToInt64(d.Key).ToString() +
						  " and location = " + warehouse.ToString();

					i_up = new SqlRequest.SQLRequest(SqlRequest.StaticServer.CaseyH, req);

					i_up.Invoke();
				}

				foreach(DictionaryEntry d in p.musicList)
				{
					int currentQty = GetCurrentQuantity(Convert.ToInt64(d.Key), warehouse, Mode.MUSIC);
					MusicCartItem b = d.Value as MusicCartItem;

					int NewQty = currentQty - b.Quantity;
					
					req = "UPDATE music_inv\nset availible = " + NewQty.ToString() + " where musicid = " + Convert.ToInt64(d.Key).ToString() +
						  " and location = " + warehouse.ToString();

					i_up = new SqlRequest.SQLRequest(SqlRequest.StaticServer.CaseyH, req);

					i_up.Invoke();
				}

			}			
		}

		
		private int GetCurrentQuantity(Int64 ID, int warehouse, Mode m)
		{
			string req;
			SqlRequest.SQLRequest i_up;

			if(m == Mode.BOOKS)
				req = "select availible from book_inv where bookid = " + ID.ToString() + " and location = " + warehouse.ToString();
			else
				req = "select availible from music_inv where musicid = " + ID.ToString() + " and location = " + warehouse.ToString();

			i_up = new SqlRequest.SQLRequest(SqlRequest.StaticServer.CaseyH, req);

			DataSet d = i_up.Invoke();

			if(d.Tables[0].Rows.Count != 0)
				return Convert.ToInt32(d.Tables[0].Rows[0]["availible"]);
			else
				return -1;
		}

		
        
		#endregion
				
	}
}
