using System;
using System.Xml;
using System.Collections;
using System.IO;

namespace foofaraw.transactions
{
    class XMLCreator
    {
        private XmlDocument document;

        public XMLCreator() 
        {
            document = new XmlDocument();
        }
   
        public void CreateOrderXML(Package p, Transaction t) 
        {
            //InventoryXML inventory = new InventoryXML(document);
            //ShippingXML shipping = new ShippingXML(document);
            
            //add the root element
            XmlElement root = document.CreateElement("","order","");

            // Add company name
            root.AppendChild(this.new_element("issuingCompany","foofaraw"));

            // Add company name
            root.AppendChild(this.new_element("partnerCompany",p.companyName));
            
            XmlElement product;

            foreach(DictionaryEntry de in p.bookList)
            {
                BookCartItem bc = de.Value as BookCartItem;

                product = document.CreateElement("", "product", "");
 
                product.AppendChild(this.new_element("type","book"));

                product.AppendChild(this.new_element("identifier", bc.ItemID));

                product.AppendChild(this.new_element("name", bc.Title));

                product.AppendChild(this.new_element("quantity", bc.Quantity.ToString()));

                root.AppendChild(product);
            }

            foreach(DictionaryEntry de in p.musicList)
            {
                MusicCartItem mc = de.Value as MusicCartItem;
                
                product = document.CreateElement("", "product", "");
 
                product.AppendChild(this.new_element("type","music"));

                product.AppendChild(this.new_element("identifier", mc.ItemID));

                product.AppendChild(this.new_element("name", mc.Album));

                product.AppendChild(this.new_element("quantity", mc.Quantity.ToString()));

                root.AppendChild(product);
            }

			
			root.AppendChild(this.new_element("shippingMethod",p.chosenOptionMethodID.ToString()));
			
			XmlElement destination, contact;
			destination = document.CreateElement("","destination","");
			destination.AppendChild(this.new_element("address", t.sAddress.ToString()));
			destination.AppendChild(this.new_element("city", t.sCity.ToString()));
			destination.AppendChild(this.new_element("state", t.sStateID.ToString()));
			destination.AppendChild(this.new_element("zip", t.sZip.ToString()));
           		
			contact = document.CreateElement("", "contact", "");
			contact.AppendChild(this.new_element("tel", t.sPhone.ToString()));
			contact.AppendChild(this.new_element("fax", "1-800-fax-them"));
			
			destination.AppendChild(contact);
			root.AppendChild(destination);

			
            

            document.AppendChild(root);
        }


        public void SaveDocument(string filename) 
        {
            try
            {                
                //XmlTextWriter x = new XmlTextWriter(filename, null);
                //x.Formatting = Formatting.Indented;

                //document.Save(filename);
				
				StreamWriter output = new StreamWriter(filename);
				output.Write(document.InnerXml);
				output.Close();
				
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }
        }


        //used so we can use the same XMLCreator for every package
        //only call this AFTER you have already saved the document, obviously
        public void ClearDocument()
        {
            this.document = new XmlDocument();
        }

     
        private XmlElement new_element(string tagname, string tagvalue) 
        {
            XmlElement element = document.CreateElement("",tagname,"");
            element.AppendChild(document.CreateTextNode(tagvalue));
            return element;
        }

            
    }
}