using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using foofaraw.management;

namespace foofaraw
{
	/// <summary>
	/// Summary description for warehouse_detail.
	/// </summary>
	public class warehouse_detail : System.Web.UI.Page
	{
		protected System.Web.UI.WebControls.Label warehousename;
		protected System.Web.UI.WebControls.Label info;
		protected System.Web.UI.WebControls.DataGrid bookinventory;		
		protected System.Web.UI.WebControls.DataGrid musicinventory;
		protected System.Web.UI.HtmlControls.HtmlForm Form1;
		
		private int id;
		private ManagementSearch search;

		private void Page_Load(object sender, System.EventArgs e)
		{
			search = new ManagementSearch();
			string id_string = Request.QueryString["id"];	
			if(id_string != null) 
			{		
				this.id = int.Parse(id_string);
			}

			if(!Page.IsPostBack)		
			{				
				if(id_string != null) 
				{
					PopulateInfo();
				}
			} 
		}

		private void PopulateInfo() 
		{
			DataTable warehouse = search.GetCurrentWarehouseInfo(this.id).Tables[0];
			DataTable vendors = search.GetVendorsForWarehouse(this.id).Tables[0];
			DataTable shipped = search.GetNumPackagesShippedFromWarehouse(this.id).Tables[0];
            if(warehouse.Rows.Count == 0 || vendors.Rows.Count == 0	)
				return;
			warehousename.Text = warehouse.Rows[0]["city"].ToString();
			FillInfoBox(warehouse.Rows[0],vendors,shipped.Rows[0]);
			bindBooks();
			bindMusic();
		}

		private void bindBooks() 
		{
			bookinventory.DataSource = this.search.GetBookInventoryAtWarehouse(this.id);
			bookinventory.DataBind();
			bookinventory.Visible = true;
		}

		private void bindMusic() 
		{
			musicinventory.DataSource = this.search.GetMusicInventoryAtWarehouse(this.id);
			musicinventory.DataBind();
			musicinventory.Visible = true;
		}

		private void FillInfoBox(DataRow warehouse, DataTable vendors, DataRow shipped) 
		{
			int packages = 0;
			if(!(shipped["total"] is DBNull))
				packages = (int)shipped["total"];

			info.Text = warehouse["address"] + ", " + warehouse ["city"] + " " + warehouse["zipcode"] + "<br />";
			info.Text += "Manager: " + warehouse["manager"] + "<br />";
			info.Text += "Phone: " + warehouse["telephone"] + "<br />";
			info.Text += "Fax: " + warehouse["fax"] + "<br /><br />";
			info.Text += "Vendors for this Warehouse: ";
			foreach(DataRow row in vendors.Rows) 
			{
				info.Text += row["vendor_name"] + ", ";
			}
			info.Text = info.Text.Remove(info.Text.LastIndexOf(","),2);
			info.Text += "<br />";
			info.Text += "Packages shipped from this location: " + packages;
		}

		private void bookinventory_PageIndexChanged(object sender, DataGridPageChangedEventArgs e) 
		{
			bookinventory.CurrentPageIndex = e.NewPageIndex;
			bindBooks();
		}

		private void musicinventory_PageIndexChanged(object sender, DataGridPageChangedEventArgs e) 
		{
			musicinventory.CurrentPageIndex = e.NewPageIndex;
			bindMusic();
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.bookinventory.PageIndexChanged += new System.Web.UI.WebControls.DataGridPageChangedEventHandler(this.bookinventory_PageIndexChanged);
			this.musicinventory.PageIndexChanged += new System.Web.UI.WebControls.DataGridPageChangedEventHandler(this.musicinventory_PageIndexChanged);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
